
/* 
 * CycleView.m -- Implementation file for the CycleView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "CycleView.h"
#import "drawingFuncs.h"
#import "nxlcau21.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>
#include <stdio.h>
#include <time.h>

int    bdry, lbou, rbou;	/* boundary cells in cycle diagram	*/
int    cysh;			/* cycle marking option			*/
int    cyno;			/* node number in cycle diagram	    	*/
int    cc, cy; 			/* number of points in cycle array 	*/
int    xhigh, nhigh;		/* maximum height, minimum height  	*/
int    xroot, nroot;		/* maximum cycle, minimum cycle    	*/
int    cynum;			/* number of cycles			*/
int    thyscom;			/* component for mousedown search	*/
int    dclik;			/* double-click option			*/
int    labop;			/* labelling option			*/
int    mdrag;			/* mousedrag option			*/
int    shopt;			/* show option				*/

int    cyback=YES;		/* draw a background for the view	*/

int    cylink[XCY];		/* links in the cycle diagram		*/
int    cynanc[XCY][XAN];	/* no of ancestors in the cycle diagram	*/
int    cyhanc[XCY];		/* ancestor histogram for cycles	*/
int    cyhigh[XCY];		/* level of cycle diagram nodes		*/
int    cymass[XCY];		/* number of nodes in node's own tree	*/
int    cyroot[XCY];		/* root for each node in cycle diagram	*/
int    cycomp[XCY];		/* component for node in cycle diagram	*/
int    cylcom[XCY];		/* list of components in cycle diagram	*/
int    cyarry[XCY];		/* scratch array for cycle diagrams	*/
double ccntr[XCY][4];		/* coordinates of cycle diagram nodes	*/
double angl, bond, disk;	/* bond angle, length, disk radius	*/

char    cyclch[WNML], timday[30];
struct  tm  *timeptr;
time_t  secsnow;

extern char   arule[DS], arr1[AL], arr2[AL], arr3[AL];

/* nxcycl3h.h					*/
/* program for LCA3H option 'd'			*/
/* display cycles up to maximum array size 	*/

void  asctobin();
int   closeto(double x, double y);
int   cofmcy(double *x, double *y, int k);
void  cycle(int cy);
void  edcyl(int *arry, int cc, int k, int l, int cn, int m);
void  finderset(int *e, twoparn *t);
void  geanc(int ance[][XAN], int *link, int cp);
void  gecyl(int *arry, int cy, int bl, int br, int l);
void  gehei(int *high, int *root, int *link, int cp);
void  geroo(int *root, int *comp, int *link, int cp);
void  hianc(int *anhi, int *ance, int cp);
void  hiroo(int *rohi, int *root, int cp);
void  howdown(int *sud);
void  ingather(double clst[][4], int *arry, int k, int n);
int   level (int n);
int   nearathand(double x, double y, int c);
int   nearby(double x, double y);
void  niceroot(int root, double x, double y, double r, double s);
void  nicetree(int root, int k, double x0, double y0, double r,
	 double th, double dt);
void  onegen(int n);
void  outgather(double clst[][4], int *arry, int k);
int   prnins(int i, twoparn *z);
int   prouts(int i, twoparn *z);
void  prxtrmd(int *min, int *max);
void  prxtrml(int *min, int *max);
int   putlevel(twoparn *zmat, twoparn *xmat, int n);
int   shflev(int g, int s);
void  suanc();
void  upthere(int *sud);

/* end nxcycl3h.h */

@implementation CycleView

/* --------------------- B R O W S E R   D E L E G A T E ------------------- */


- (int)browser: sender fillMatrix: matrix inColumn: (int)col {
char ch[48];
int  i, j, k, l, m;
        if (col==0 && cynum==0) {
        [matrix addRow];
        [[matrix cellAt: 0: 0] setLeaf: YES];
	[[matrix cellAt: 0: 0] setLoaded: YES];
        [[matrix cellAt: 0: 0] setEnabled: NO];
	[[matrix cellAt: 0: 0] setStringValue: "choose a cycle length"];
	return 1;
	} else if (col==0) {
	  for (i=0; i<cynum; i++) {
            [matrix addRow];
            [[matrix cellAt: i: 0] setLeaf: YES];
	    [[matrix cellAt: i: 0] setLoaded: YES];
            [[matrix cellAt: i: 0] setEnabled: YES];
  l=k=cylcom[i]; j=m=0;
  do {l=cylink[l]; j++; m+=cymass[l];} while (l!=k);
  sprintf(ch,"%3d - rootcycle length %3d,  total mass %5d",i,j,m);
            [[matrix cellAt: i: 0] setStringValue: ch];
            }
	  return cynum;
	  }
return 0;
}

/* to distract the single click in a browser double click */
- nhul: sender {
  thyscom=[[sender matrixInColumn: 0] selectedRow];
  cyno=cylcom[thyscom];
  [[rootWindows objectAt: thyscom] orderFront: self];
  [[rootViews objectAt: thyscom] display];
 return self;}

/* - - - - - - - -   F I L E   A N D   D I R E C T O R Y   - - - - - - - - */

- setFilename:(const char *)aFilename {
if (filename) free(filename);
filename = malloc(strlen(aFilename)+1);
strcpy(filename, aFilename);
[window setTitleAsFilename:aFilename];
return self;
}

- openCycl:sender {
  char *types[2] = {"cyl",0};
  int  i, fd;
  NXStream *theStream;

// printf("hello from openCycl\n");
  [[OpenPanel new] allowMultipleFiles: NO];
  [[OpenPanel new] chooseDirectories: NO];

  if ([ [OpenPanel new] runModalForTypes:types]) {
    [self setFilename: [[OpenPanel new] filename]];
    fd = open(filename, O_RDONLY, 06666);
    theStream = NXOpenFile(fd, NX_READONLY);
//      NXScanf(theStream,"%s",cstr);
      i=0;
      while (NXScanf(theStream,"%6lf,%6lf,%6lf,%6lf",
	&ccntr[i][0],&ccntr[i][1],&ccntr[i][2],&ccntr[i][3])!=EOF) i++;
    NXClose(theStream);
    close(fd);
    [window setTitleAsFilename: filename];
    [window setDocEdited:NO];
//  [xtrnWin setSel: 0: [xtrnWin textLength]];
//  [xtrnWin replaceSel: cstr];
//  [xtrnWin scrollSelToVisible];
//  [xtrnWin display];
    }

  return self;
}

- saveCyclAs:sender {
  id panel;
  const char *dir;
  char *file;

/* prompt user for the file name and save to that file */

  if (filename==0) {
    /* no filename; set up defaults */ 
    dir  = NXHomeDirectory();
    file = (char *)[window title];
    } else {
    file=rindex(filename,'/');
    if (file) {
      dir   = filename;
      *file = 0;
      file++;
      } else {
      dir  = filename;
      file = (char *)[window title];
      }
    }
  panel = [SavePanel new]; 
  [panel setRequiredFileType: ""];
  if ([panel runModalForDirectory: dir file: file]) {
    [self setFilename: [panel filename]];
    return [self saveCycl: sender];
    }
    return nil;  /* didn't save */
  }
    
- saveCycl:sender {
  int      i, fd;
  NXStream *theStream;

  if (filename==0) return [self saveCyclAs: sender];
  [window setTitle: "Saving ..."];

  fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 06666);
  if (fd < 0) {
    NXRunAlertPanel(0, "Cannot save file: %s",0,0,0,strerror(errno));
    return self;
    }
  theStream = NXOpenFile(fd, NX_WRITEONLY);
      for (i=0; i<XCY; i++)
	NXPrintf(theStream,"%4.2f,%4.2f,%4.2f,%4.2f\n",
	  ccntr[i][0],ccntr[i][1],ccntr[i][2],ccntr[i][3]);

  NXClose(theStream);
  close(fd);
//  [window setTitleAsFilename: filename];
//  [window setDocEdited:NO];
// printf("saveCycl ended\n");
  return self;
}

- close:sender {const char *fname;
int         q;
  if ([window isDocEdited])
//    fname=filename ? filename : [sender title];
//    if (rindex(fname,'/')) fname = rindex(fname,'/') + 1;
    q = NXRunAlertPanel(
      "Save",
      "Save changes to %s?",
      "Save",
      "Don't Save",
      "Cancel",
      filename);
    if (q==1) {if (![self saveCycl:nil]) return nil;}
    if(q==-1) return nil;
//  [sender setDelegate: nil];
//  [proc free];
//  [self free];
  return self;
}

- windowWillClose:sender {
[self close: self];
// printf("windowWillClose\n");
return self;
}

/* -------- displaying, printing, and copying a tree to a pasteboard ------ */

/* display the graph on the console screen */

- drawCycle:sender {[self display]; return self;}

- windowDidBecomeKey:sender {
   if (sender==cycleWindow) {
	[menuCopy setTarget: self];
        [menuCopy setAction: @selector(copyCycle:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printCycle:)];
	[menuPrint setEnabled: YES];
     } else {
	thyscom=[rootWindows indexOf: sender];
	[[browser matrixInColumn: 0] selectCellAt: thyscom: 0];
	[browser sendAction];
     }
return self;
}

- windowDidResignKey:sender {
 	[menuCopy setEnabled: NO];
 	[menuPrint setEnabled: NO];
return self;
}

/* display the graph on the printer */

- printCycle:sender {int i;
  i=[[browser matrixInColumn: 0] selectedRow];
  [[rootWindows objectAt: i] smartPrintPSCode: self];
  return self;
}

- copyCycle:sender {int i;
NXRect zbounds;
id     pBoard;
  cyback=NO;
  i=[[browser matrixInColumn: 0] selectedRow];
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [[rootViews objectAt: i] getBounds: &zbounds];
  [[rootViews objectAt: i] writePSCodeInside: &zbounds to: pBoard]; 
  cyback=YES;
return self;
}

/* -------------------------------- C Y C L -------------------------------- */

/* put up the principal window for the CycleView. Since CycleView.nib is a */
/* secondary .nib file, it is not loaded until there is a demand, which    */
/* would usually arise from the "Cycle" option in the main menu.	   */

- showCycle:sender {
  if (cycleWindow==nil) {
    if (![NXApp loadNibSection: "cycle.nib" owner: self withNames: NO]) 
	{return nil;}
    rootViews = [[List alloc] init];
    rootWindows = [[List alloc] init];
    [cyWidth setEmptySelectionEnabled: YES];
    [cyWidth selectCellAt: -1: 0];
    [browser setEmptySelectionEnabled: YES];
    [browser setMultipleSelectionEnabled: NO];
    [browser getTitleFromPreviousColumn: NO];
    [browser setTitle: "components" ofColumn: 0];
    [self setGraphData];
    }
  [cycleWindow orderFront: nil];
  wru(cyclch);
  [ruleNumber setStringValue: cyclch];
  [nodeNumber setIntValue: 0];
  [self ediOpt: self];
  return self;
  }

- cySample:sender {int i, j, n;
  for (i=0; i<AL/cy; i++) {
    n=cyno;
    for (j=0; j<cy; j++) {arr1[cy*i+(cy-j-1)]=n%KK; n/=KK;}
    }
  [evWindow makeKeyAndOrderFront: sender];
  [evButton performClick: sender];
  return self;
}

/* read all the relevant parameters, construct the auxiliary tables,	*/
/* including an initial random assignment of coordinates to the nodes,	*/
/* and display the graph. This method will usually be invoked selecting	*/
/* the cycle length.							*/

- prepareCycle:sender {
char   c[40];
int    i, j;
  wru(cyclch);
  [ruleNumber setStringValue: cyclch];
  if (cynum>0) {
    for (i=0; i<cynum; i++) [[rootWindows objectAt: i] setFreeWhenClosed: YES];
    [rootWindows makeObjectsPerform: @selector(close)];
    [rootWindows empty];
    [rootViews empty];
    }
   bdry=0;
   cyno=0;
   cysh=XCY;
    switch([sender selectedCol]) {
      case 0:  cc=4;    cy=2; break;
      case 1:  cc=8;    cy=3; break;
      case 2:  cc=16;   cy=4; break;
      case 3:  cc=32;   cy=5; break;
      case 4:  cc=64;   cy=6; break;
      case 5:  cc=128;  cy=7; break;
      case 6:  cc=256;  cy=8; break;
      case 7:  cc=512;  cy=9; break;
      case 8:  cc=1024; cy=10; break;
      case 9:  cc=2048; cy=11; break;
      case 10: cc=4096; cy=12; break;
      default: cc=64;   cy=4; break;}
   [nodeNumber setStringValue: ""];

        if (strcmp([[cyBou selectedCell] title],"cycle")==0) bdry=0;
        if (strcmp([[cyBou selectedCell] title],"fixed")==0) bdry=1;
	lbou=[leftBoundary selectedRow];
	rbou=[rightBoundary selectedRow];
        gecyl(cylink,cy,lbou,rbou,bdry);
	geanc(cynanc,cylink,cc);
	geroo(cyroot,cycomp,cylink,cc);
	hiroo(cymass,cyroot,cc);
	gehei(cyhigh,cyroot,cylink,cc);
	
	[[cyTriv1 cellAt: 0: 0] setIntValue: xhigh];
	[[cyTriv1 cellAt: 1: 0] setIntValue: nhigh];
	[[cyTriv1 cellAt: 2: 0] setIntValue: cynum];
	[[cyTriv1 cellAt: 3: 0] setIntValue: xroot];
	[[cyTriv1 cellAt: 4: 0] setIntValue: nroot];

  for (i=0; i<cynum; i++) {
    [oneRoot oneRoot: self];
    [rootWindows addObject: [oneRoot whoWindow: self]];
    [rootViews addObject: [oneRoot whoView: self]];
    sprintf(c,"component number %2d",i);
    [[rootWindows lastObject] setTitle: c];
    [[rootWindows lastObject] setDelegate: self];
    [[rootViews lastObject] setDelegate: self];
    [[rootViews lastObject] thouArt: i];
    niceroot(cylcom[i],0.0,0.0,161.8*bond,150.0*bond);
    }
  [browser loadColumnZero];
  return self;
}

- awakeFromNib:sender {printf("awake Cycleview\n"); return self;}

- markRadius:sender {
  bond=[sender doubleValue];
  [radSlid setDoubleValue: bond];
  [radText setDoubleValue: bond];
  return self;
}

- markAngle:sender {
  angl=[sender doubleValue];
  [angSlid setDoubleValue: angl];
  [angText setDoubleValue: angl];
  return self;
}

- markDysk:sender {
  disk=[sender doubleValue];
  [dskSlid setDoubleValue: disk];
  [dskText setDoubleValue: disk];
  return self;
}

- setGraphData {
  [radSlid setDoubleValue: 0.282];
  [radText setDoubleValue: 0.282];
  [angSlid setDoubleValue: 0.282];
  [angText setDoubleValue: 0.282];
  [dskSlid setDoubleValue: 0.282];
  [dskText setDoubleValue: 0.282];
  angl=0.282;
  disk=0.282;
  bond=0.282;
return self;
}

- makeTree:sender {int j;
  j=cyroot[cyno];
  niceroot(j,240.0,240.0,25.0,20.0);
  [self display];
return self;
}

/* draw the cycle window supposing that none of the basic parameters 	*/
/* such as cycle width or boundary condition have changed, but that the */
/* view needs cleaning or that the style of presentation has changed.	*/

- redrawCycle:sender {
  [self setCyTriv];
  [self ediOpt: self];
  if (thyscom<[rootViews count]) [[rootViews objectAt: thyscom] display];
  return self;
}

/* editing options, including scope of mouse dragging, reaction to double */
/* clicks, style of presentation of the graph.				  */

- ediOpt:sender {
  dclik=[moveWhich selectedRow];
  labop=[labelStyle selectedRow];
  mdrag=[dragStyle selectedRow];
  shopt=[treeShow selectedRow];
return self;
}

- locAte:sender {int i;
  i=[nodeNumber intValue];
  if (i<XCY) {
    [nodeNumber setIntValue: cysh];
    if (strcmp([[cyNode selectedCell] title],"self")==0) cysh=i;
    if (strcmp([[cyNode selectedCell] title],"next")==0) cysh=cylink[i];
    if (strcmp([[cyNode selectedCell] title],"none")==0) cysh=XCY;
    if (strcmp([[cyNode selectedCell] title],"root")==0) cysh=cyroot[i];
    [nodeNumber setIntValue: cysh];
    if (cysh<XCY) for (i=0; i<cynum; i++) if (cylcom[i]==cycomp[cysh]) break;
    if (i<cynum) [[browser matrixInColumn: 0] selectCellAt: i: 0];
    [browser sendAction];
    } 
  else [nodeNumber setIntValue: XCY];
return self;
}


/* fill the cycle trivia panel with information pertinent to the graph.	*/
/* The upper part of the panel holds generic information concerning the	*/
/* graph, the lower part, specific properties of the selected node.	*/

- setCyTriv {int i, j, k, m;
  [nodeNumber setIntValue: cyno];
  [[cyTriv2 cellAt: 0: 0] setIntValue: cyno];
  [[cyTriv2 cellAt: 1: 0] setIntValue: cyhigh[cyno]];
  j=cynanc[cyno][0];
  if (cyroot[cyno]==cyno) j--;
  [[cyTriv2 cellAt: 2: 0] setIntValue: j];
  [[cyTriv2 cellAt: 3: 0] setIntValue: cymass[cyroot[cyno]]];
  i=k=cyroot[cyno]; j=m=0;
  do {i=cylink[i]; j++; m+=cymass[i];} while (i!=k);
  [[cyTriv2 cellAt: 4: 0] setIntValue: j];
  [[cyTriv2 cellAt: 5: 0] setIntValue: m];
  return self;
}

- cyclMove:sender 
{char *c; int i, n; double p, th, x, y, t, x0, y0;
  c=[[sender selectedCell] title];
// printf("hello from cyclMove %s in %2d\n",c,thyscom);
         if (strcmp(c,"dilate")==0) {
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    ccntr[i][0]*=1.125;
	    ccntr[i][1]*=1.125;
	    }
  } else if (strcmp(c,"shrink")==0) {
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    ccntr[i][0]/=1.1667;
	    ccntr[i][1]/=1.1667;
	    }
  } else if (strcmp(c,"save")==0) {
	  for (i=0; i<XCY; i++) {
		ccntr[i][2]=ccntr[i][0];
		ccntr[i][3]=ccntr[i][1];
		}
  } else if (strcmp(c,"restore")==0) {
	  for (i=0; i<XCY; i++) {
		ccntr[i][0]=ccntr[i][2];
		ccntr[i][1]=ccntr[i][3];
		}
  } else if (strcmp(c,"left")==0) {
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    ccntr[i][0]-=15.0;
	    }
  } else if (strcmp(c,"right")==0) {
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    ccntr[i][0]+=15.0;
	    }
  } else if (strcmp(c,"up")==0) {
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    ccntr[i][1]-=15.0;
	    }
  } else if (strcmp(c,"down")==0) {
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    ccntr[i][1]+=15.0;
	    }
  } else if (strcmp(c,"rotate")==0) {
//	  th=6.28*(p-0.5);
	  th=0.314;
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    x=ccntr[i][0]; y=ccntr[i][1];
	    ccntr[i][0]= x*cos(th)+y*sin(th);
	    ccntr[i][1]=-x*sin(th)+y*cos(th);
	    }
  } else if (strcmp(c,"horiz")==0) {
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    ccntr[i][0]=-ccntr[i][0];
	    }
  } else if (strcmp(c,"vert")==0) {
	  for (i=0; i<cc; i++) if (cycomp[i]==cylcom[thyscom]) {
	    ccntr[i][1]=-ccntr[i][1];
	    }
  }
  [[rootViews objectAt: thyscom] display];
  return self;
}

/* - - - - - - - - - - - -    C Y C L E V I E W   - - - - - - - - - - - - - */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new CycleView object.  First, an initFrame: message is sent 
 * to super to initialize CycleView as a View.  Next, the CycleView sets its 
 * own state -- that it is opaque and that the origin of its coordinate system 
 * lies in the center of its area.  It then creates and initializes its 
 * associated objects, a Storage object, an NXCursor, and a Cell.  Finally, it 
 * loads into the Window Server some PostScript procedures that it will use in 
 * drawing itself. 
 */
{
	NXPoint	spot;
	
  [super initFrame: frameRect];
  [self translate: floor(frame.size.width/2): floor(frame.size.height/2)];
  [self setOpaque: YES];
  crossCursor = [NXCursor newFromImage: [NXImage 
		newFromSection:"cross.tiff"]];
  spot.x = spot.y = 7.0;
  [crossCursor setHotSpot: &spot];

//  loadPSProcedures();

//  PSsetgray(NX_BLACK);
//  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;
}

- setDelegate:anObject
/*
 * Sets the CycleView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the CycleView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
	[super sizeTo: width: height];
	[self setDrawOrigin: -floor(width/2): -floor(height/2)];
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the CycleView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although CycleView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	
	if (rects == NULL) return self;

	/* 
	 * If we're printing, we need to load drawing procedures to the 
	 * printing context.  This simple test is OK for a View that prints a 
	 * single page.  If there were multiple pages, the procedures would be 
	 * loaded multiple times.  Although this is inefficient, it causes no 
	 * error.  Views that print multiple pages should load procedures such 
	 * as these in an endPrologue method (see the View spec sheet). 
	 */

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (cyback) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(timday,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);

/* for the cycle diagram --- - - - - - - - - - - - - - - - - - - - - - - - - */

  PSsetgray(NX_BLACK); 
  if (cyback) {
    wru(cyclch);
    [ruleNumber setStringValue: cyclch];
    [[Font newFont: "Courier-Bold" size: 18.0 matrix: NX_FLIPPEDMATRIX] set];
    PSmoveto(-floor(frame.size.width/2)+10.0,
	 	-floor(frame.size.height/2)+18.0);
    PSshow(cyclch);
    PSstroke();
    }
  [[Font newFont: "Courier" size: 10.0 matrix:NX_FLIPPEDMATRIX] set];
	edcyl(cylink,
	      cc,
	      labop,
	      cysh,
	      cyno,
	      shopt);
	cysh=XCY;
  [timeDate setStringValue: timday];

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

return self;
}

- mouzeDown:(int)component {thyscom=component; return self;}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the CycleView.  The CycleView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the CycleView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the CycleView, the CycleView 
 * registers the point and then sends a message to its delegate notifying it of 
 * the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;

        double  x, y, ex, wy, th;
	int     i, j;

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];

/* for the cycle diagram --- - - - - - - - - - - - - - - - - - - - - - - - - */

 			[window setDocEdited:YES];
		        aPoint = theEvent->location;
		        [self convertPoint:&aPoint fromView:nil];
                        ex=aPoint.x;
                        wy=aPoint.y;
			cyno=nearathand(ex,wy,cylcom[thyscom]);
			[nodeNumber setIntValue: cyno];
			[delegate setCyTriv];

       if (theEvent->data.mouse.click==3) {} else 
       if (theEvent->data.mouse.click==2) {
	 for (i=0, j=1; i<cy; i++) j*=KK;
	   switch (dclik) {
	   case 0: ingather(ccntr,cylink,cyno,j); break;
	   case 1: ingather(ccntr,cylink,cylink[cyno],j);
		   ccntr[cyno][0]=ex;
		   ccntr[cyno][1]=wy;
		   break;
	   case 2: outgather(ccntr,cylink,cyno); break;
	   case 3: outgather(ccntr,cylink,cylink[cyno]);
		   ccntr[cyno][0]=ex;
		   ccntr[cyno][1]=wy;
		   break;
	   case 4: ingather(ccntr,cylink,cyno,j);
		   outgather(ccntr,cylink,cyno);
		   break;
	   default: break;}
   [self display];

		} else {
	[self lockFocus];
        PSsetinstance(YES);
	do {
 	    PSnewinstance();
	    aPoint = theEvent->location;
	    [self convertPoint:&aPoint fromView:nil];
            ex=aPoint.x;
            wy=aPoint.y;
		PSsetgray(NX_BLACK);
		PSmoveto(ex+12.0,wy);
		PSarc(ex, wy, 12.0, 0.0, 360.0);
		PSstroke();
		ccntr[cyno][0]=ex;
		ccntr[cyno][1]=wy;
	    [window flushWindow];
	    if (theEvent->type == NX_MOUSEUP) looping = NO;
   	    } while (looping && (theEvent=[NXApp getNextEvent:
		NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];
	switch (mdrag) {
	  case 0:  break;
	  case 1:
		if (cyroot[cyno]==cyno) break;
		x=ex-ccntr[cylink[cyno]][0];
		y=wy-ccntr[cylink[cyno]][1];
		th=atan2(y,x);
		nicetree(cyno,1,ex,wy,161.8*bond,th,angl);
		break;
	  default: break;}
	[self display]; 
		}

	[window setEventMask:oldMask];
	return self;
}

@end

/* - - - - -   N X C Y C L 2 1 . C   - - - - - - - - - - - - - - - - - - - - */

/* nxcycl21.c						*/
/* program for LCA21 option 'd'				*/
/* display de Bruijn diagram - 1st, 2nd, & 3rd gen	*/
/* Harold V. McIntosh, 20 February 1988			*/
/* 21 February 1988 - add j0=i0; to Pass 4's		*/
/* 26 September 1989 - show evolution of periodic conf	*/
/* 2 October 1989 - cycl21.c incorporated into rijn21.c	*/
/* 20 November 1989 - correct loop in kwait		*/
/* 22 June 1994 - adapt to NeXT				*/
/* 21 June 1995 - separated cycle from de Bruijn	*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1994		*/
/*    Copyright (C) 1995		*/
/*    Harold V. McIntosh		*/

/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

/* CYCLE21.C				  	    */

/* Pass 1 makes arry[i] equal to the successor of i */
/* the configuration is supposed to be cyclic	    */

void ylpass1(cy,cp,arry) int cy, cp; unsigned int *arry; {int i, j, x;
asctobin();
for (j=0; j<cp; j++) {
  x=j; for (i=0; i<cy; i++) {arr1[cy-i-1]=x%KK; x/=KK;}
  onegen(cy);
  x=0; for (i=0; i<cy; i++) x=KK*x+arr1[i]; arry[j]=x;
  }
}

/* zylpass 1 makes arry[i] equal to the successor of i 		*/
/* the configuration is supposed to have fixed boundary cells 	*/
/* cy = length of configuration (max CCY)			*/
/* cp = KK^cy = number of configurations of length cy (max XCY)	*/
/* bl = state of left boundary cell				*/
/* br = state of right boundary cell				*/

void zylpass1(cy,cp,bl,br,arry) int cy, cp, bl, br; unsigned int *arry; {
int i, j, x;
asctobin();
for (j=0; j<cp; j++) {
  arr1[0]=bl;
  x=j; for (i=1; i<cy+1; i++) {arr1[cy-i-1]=x%KK; x/=KK;}
  arr1[cy+1]=br;
  onegen(cy+2);
  x=0; for (i=1; i<cy+1; i++) x=KK*x+arr1[i]; arry[j]=x;
  }
}

/* edit the Pass 4 and Pass 5 cycle displays 	*/
/*   arry = linkage list			*/
/*   cc   = number of cycles			*/
/*   k    = label style (self, hight, root, &c)	*/
/*   l    = node to emphasize by heavy circle	*/
/*   cn   = reference node w/r/t m		*/
/*   m    = option for suppressing nodes	*/

void edcyl(arry,cc,k,l,cn,m) int *arry, cc, k, l, cn, m; {
int    i, j;
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, t;
  if (cc>XCY) return;
  r=36.4*disk;
  for (i=0; i<cc; i++) {

	j=arry[i];
	if (m==0 && cyroot[i]!=cyroot[cn]) continue;	/* tree      */
	if (m==1 && cyhigh[i]>0) continue;		/* roots     */
	if (m==2 && cycomp[i]!=cycomp[cn]) continue;	/* component */
	ex=ccntr[i][0];
	wy=ccntr[i][1];
	PSmoveto(ex,wy);
	x=ccntr[j][0];
	y=ccntr[j][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	eks=x-r*cos(t);
	wye=y-r*sin(t);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSmoveto(x+r,y);
	PSarc(x, y, r, 0.0, 360.0);

   ex=ccntr[i][0];
   wy=ccntr[i][1];
   PSmoveto(ex-10.0,wy+5.0);
   switch (k) {
     case 0:  sprintf(s,"%4d",i); break;
     case 1:  sprintf(s,"%4d",cyroot[i]); break;
     case 2:  sprintf(s,"%4d",cyhigh[i]); break;
     case 3:  sprintf(s," "); break;
     default: sprintf(s," "); break; }
   PSshow(s);

   }

if (l<XCY) {
	PSstroke();
	x=ccntr[l][0];
	y=ccntr[l][1];
	PSsetlinewidth(3.0);
	PSmoveto(x+r,y);
	PSarc(x, y, 0.8*r, 0.0, 360.0);
        }
PSstroke();
PSsetlinewidth(0.0);
}

/* arrange the nodes of a tree in an orderly fashion	*/
/* root = index of the root node			*/
/* k = 0 for single root, 1 otherwise		 	*/
/* (x0,y0) = proposed center of root node		*/
/* r = internode distance in the new tree		*/
/* th = angular orientation of the trunk		*/
/* dt = angular separation between branches		*/

void nicetree(root,k,x0,y0,r,th,dt) int root, k; double x0, y0, r, th, dt; {
int i, j, hj, rt; double eye, jay;
// printf("nicetree %3d\n",root);
  ccntr[root][0]=x0;
  ccntr[root][1]=y0;
  if (cynanc[root][0]==0) return;
  j=cynanc[root][0];
  hj=(j%2==0)?j/2:(j-1)/2;
  jay=dt*((double)(hj));
  for (i=2; i<cynanc[root][1]; i++) {
	rt=cynanc[root][i];
	if (cyroot[rt]==rt) continue;
	eye=(double)(i-hj);
	nicetree(rt,
		 1,
		 x0+r*cos(th+eye*dt),
		 y0+r*sin(th+eye*dt),
		 1.1*r,
		 th+eye*dt,
		 (k==0)?0.25:0.95*dt);
  }
}

/* display a component so that it looks neat and orderly */
/* root = index of the root node			 */
/* (x,y) = proposed center of root cycle		 */
/* r = radius of the rootcycle				 */
/* s = radial scale for the trees			 */

void niceroot(root,x,y,r,s) int root; double x, y, r, s; {
int i, j, rt, n; double ex, wy, th, dt;
  n=cofmcy(&ex,&wy,root);
  j=cynanc[root][0];
      if (n==1) nicetree(root,
		       0,
		       x,
		       y,
		       s,
		       0.0,
		       6.28/((double)(j==1?j:j-1)));
  else {
        dt=6.28/((double)n);
        for (i=0, th=0.0, rt=root; i<n; i++, th+=dt, rt=cylink[rt]) 
          nicetree(rt,
		   1,
		   x+r*cos(th),
		   y+r*sin(th),
		   s,
		   th,
		   angl);
	}
}

/* find the center of mass of the rootcycle of a component */

int cofmcy(x,y,k) double *x, *y; int k; {
int i, j, l, n; double ex, wy, en;
  j=l=cyroot[k];
  ex=0.0;
  wy=0.0;
  for (i=0, n=1; i<XCY; i++, n++) {
    ex+=ccntr[j][0];
    wy+=ccntr[j][1];
    j=cylink[j];
    if (j==l) break;
  }
  en=(double)n;
  *x=ex/en;
  *y=wy/en;
  return n;
}

/* generate the cycle linkage list */

void gecyl(arry,cy,bl,br,l) int *arry, cy, bl, br, l; {
int i, cp;
if (cy>CCY) return;
for (i=0, cp=1; i<cy; i++) cp*=KK;
  if (l==0) ylpass1(cy,cp,arry);
  if (l==1) zylpass1(cy,cp,bl,br,arry);
}

/* find index of nearest center to x, y (searching everywhere) */

int nearby(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<cc; i++) {
  ex=x-ccntr[i][0];
  wy=y-ccntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* find index of nearest center to x, y, (searching the component) */

int nearathand(x,y,c) double x, y; int c; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<cc; i++) if (cycomp[i]==c) {
  ex=x-ccntr[i][0];
  wy=y-ccntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* count the number of ancestors			*/
/* the actual ancestors are also recorded up to a 	*/
/* maximum of XAN, after which they are only counted	*/

void geanc(ance,link,cp) int ance[][XAN], *link, cp; {
int i, j, k;
for (i=0; i<cp; i++) {ance[i][0]=0; ance[i][1]=2;}
for (i=0; i<cp; i++) {
  j=link[i];
  ance[j][0]++;
  k=ance[j][1];
  if (k<=XAN) {ance[j][k]=i; ance[j][1]++;}
  }
}

/* make a histogram of ancestor counts */

void hianc(anhi,ance,cp) int *anhi, *ance, cp; {
int i;
for (i=0; i<cp; i++) anhi[i]=0;
for (i=0; i<cp; i++) anhi[ance[i]]++;
}

/* count the number of occurrences of a given root */

void hiroo(rohi,root,cp) int *rohi, *root, cp; {
int i;
for (i=0; i<cp; i++) rohi[i]=0;
for (i=0; i<cp; i++) rohi[root[i]]++;
}

/* find the root to which each node is attached     */
/* find the component to which each node belongs    */
/* each node on a cycle is the root of its own tree */
/* root[] = array where roots are recorded	    */
/* comp[] = array where components are recorded	    */
/* link[] = array where (out) links are recorded    */
/* cp     = mutual actual length of these arrays    */
/* cynum  = global counting number of cycles	    */
/* xroot  = global holding length of longest cycle  */
/* nroot  = global holding length of shortes cycle  */
/* cylcom[] = global array listing components 	    */
/* cyarry[] is a scratch array			    */

void geroo(root,comp,link,cp) int *root, *comp, *link, cp; {
int i, j, k, l;
  l=0;
  cynum=0;
  xroot=0;
  nroot=XCY;
  for (i=0; i<cp; i++) cyarry[i]=0;
  for (i=0; i<cp; i++) {
    j=i;
    if (cyarry[j]==2) continue;
    while (cyarry[j]==0) {cyarry[j]=1; j=link[j];}
    if (cyarry[j]==1) {
      k=j;
      l=0;
      cylcom[cynum]=j;
      cynum++;
      while (cyarry[k]==1) {cyarry[k]=2; root[k]=k; comp[k]=j; k=link[k]; l++;}
      }
    if (l>xroot) xroot=l;
    if (l<nroot) nroot=l;
    if (cyarry[j]==2) j=root[j];
    for (k=0; k<cp; k++) 
	if (cyarry[k]==1) {root[k]=j; comp[k]=comp[j]; cyarry[k]=2;}
    }
}

/* find the height of each node, assuming that the roots are known */

void gehei(high,root,link,cp) int *high, *root, *link, cp; {
int i, j, k, l;
  xhigh=0;
  nhigh=XCY;
  for (i=0; i<cp; i++) high[i]=XCY;
  for (i=0; i<cp; i++) high[root[i]]=0;
  for (i=0; i<cp; i++) {
    for (l=0, j=0; j<cp; j++) {
      if (j==root[j]) continue;
      k=high[link[j]]; 
      if (k<XCY) {high[j]=k+1; l=1;}
      }
    if (l==0) break;
    }
  xhigh=0;
  nhigh=XCY;
  for (i=0; i<cp; i++) {
    j=high[i];
    if (j==XCY) continue;
    if (j>xhigh) xhigh=j;
    if (j<nhigh) nhigh=j;
  }
}

/* move the in-linked nodes into close proximity */

void ingather(clst,arry,k,n) double clst[][4]; int *arry, k, n; {
int i;
double x, xx, y, yy;
  x = clst[k][0];
  y = clst[k][1];
  for (i=0; i<n; i++) if (arry[i]==k) {
    xx = clst[i][0];
    yy = clst[i][1];
    clst[i][0]=0.8*x+0.2*xx;
    clst[i][1]=0.8*y+0.2*yy;
    }
}

/* move the out-linked node into close proximity */

void outgather(clst,arry,k) double clst[][4]; int  *arry, k; {
int j;
double x, xx, y, yy;
  j=arry[k];
  x = clst[k][0];
  y = clst[k][1];
  xx = clst[j][0];
  yy = clst[j][1];
  clst[j][0]=0.8*x+0.2*xx;
  clst[j][1]=0.8*y+0.2*yy;
}

/* end nxcycl21.c */


/* end CycleView.m */
