
/* 
 * LiulGuph.m -- Implementation file for the LiulGuph class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "LiulGuph.h"
#import "nxlcau21.h"
#import "nxardn21.h"
#import "nxrijn21.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>
#include <stdio.h>
#include <time.h>

int    cnstRijn=NO;	/* YES/NO for evolve to constant/shift		*/
int    drawBasic=NO;	/* colored drawing of de Bruijn diagram itself  */
int    justRijn=NO;	/* black de Bruijn diagral of appropriate order */
int    lastDraw=0;	/* 0 = none, 1 = rijn, 2 = basic, 3 = generic   */
int    rijnBack=YES;	/* include an opaque background in the drawing  */
int    shortRijn=NO;	/* elaborate diagram whichever cnstRijn setting */

int    rijnPerm;
int    rijnShift;
int    rijnGen;

extern  parneih    amat;
extern  parneih    bmat;

extern twoparn    smat;
extern twoparn    tmat;
extern twoparn    umat;
extern twoparn    vmat;

extern NXColor colst[KK];

extern char    dbtag[DDN];	/* tag the nucleus */
extern int     dbno;		/* node number in de Bruijn diagram 	*/
extern int     dd;		/* radius of de Bruijn ring */
extern double  dcntr[][2];

void   asctobin();
int    closeto(double x, double y);
void   edcyl(int *arry, int cc, int k, int l, int cn, int m);
void   edijn(int g, int s, int p, int r);
void   finderset(int *e, twoparn *t);
void   geanc(int ance[][21], int *link, int cp);
void   gecyl(int *arry, int cy, int bl, int br, int l);
void   geroo(int *root, int *comp, int *link, int cp);
void   howdown(int *sud);
int    level (int n);
int    prnins(int i, twoparn *z);
int    prouts(int i, twoparn *z);
void   prxtrmd(int *min, int *max);
void   prxtrml(int *min, int *max);
int    putlevel(twoparn *zmat, twoparn *xmat, int n);
int    shflev(int g, int s);
void   shortevolve(int m, int n);
void   suanc();
void   upthere(int *sud);
void   wru(char *c);
void   wrru(char *c);

@implementation LiulGuph


/* -------------------------------- R I J N -------------------------------- */

- loadRijn:sender {
  if (rijnWindow==nil) {
    if (![NXApp loadNibSection: "rijn.nib" owner: self withNames: NO]) 
	{return nil;}
    lastDraw=0;
    [self rijnReset: self];
    }
  [rijnWindow makeKeyAndOrderFront: nil];
  return self;
  }

- rijn:sender {int i;
  rijnGen=    [sender selectedRow];
  rijnShift = 6-(int)[sender selectedCol];
  i=shflev(rijnGen,rijnShift);
  [self rijnReset: self];
  [sender selectCellAt: rijnGen: 6-rijnShift];
  [[underDiagram cellAt: 0: i] performClick: sender];
  [underDiagram selectCellAt: 0: i];
  rijnPerm=[whichPermutation selectedRow];
  edijn(rijnGen,rijnShift,rijnPerm,![dbBranches state]);
  cnstRijn=NO;
return self;
}

- rijnConst:sender {
  rijnGen=[sender selectedRow];
  rijnShift=[sender selectedCol];
  [self rijnReset: self];
  [sender selectCellAt: rijnGen: rijnShift];
  [[underDiagram cellAt: 0: rijnGen] performClick: self];
  [underDiagram selectCellAt: 0: rijnGen];
  rijnPerm=[whichPermutation selectedRow];
  edijn('G'-'a'+rijnGen,rijnShift,rijnPerm,![dbBranches state]);
  cnstRijn=YES;
return self;
}

- rijnReset:sender {char c[WNML];
  [whichShift setEmptySelectionEnabled: YES];
  [whichShift selectCellAt: -1: 0];
  [whichConstant setEmptySelectionEnabled: YES];
  [whichConstant selectCellAt: -1: 0];
  [underDiagram setEmptySelectionEnabled: YES];
  [[underDiagram selectedCell] setIntValue: 0];
  wru(c);
  [ruleNumber setStringValue: c];
return self;
}

- rijnHowBig:sender {int i; double th, dt;
    switch([sender selectedCol]) {
      case 0:  dd=4;    break;
      case 1:  dd=16;   break;
      case 2:  dd=64;   break;
      case 3:  dd=256;  break;
      case 4:  dd=1024; break;
      case 5:  dd=4096; break;
      default: dd=16; break;}
  dt=6.28318/((double)(dd));
  for (i=0, th=0.5*dt; i<dd; i++, th+=dt) {
    dcntr[i][0]=200.0-180.0*sin(th);
    dcntr[i][1]=200.0-180.0*cos(th);
    }

return self;
}

- rijnRanSam:sender {int  i;
  i='z';
  if (strcmp([[sender selectedCell] title],"random sample")==0) i='r';
  if (strcmp([[sender selectedCell] title],"selected node")==0) i='n';
  if (strcmp([[sender selectedCell] title],"scan links")==0)    i='i';

  if ([dbBranches state]) {return self;}
  if (cnstRijn) edshc(rijnGen,i); else edshf(rijnGen,rijnShift,i);
  [evWindow makeKeyAndOrderFront: sender];
  [evButton performClick: sender];
return self;
}

- basicRijn:sender {char c[WNML], d[37+WNML];
  drawBasic=YES;
  [self rijnReset:self];
  [[underDiagram cellAt: 0: 0] performClick: sender];
  wru(c);
  sprintf(d,"lcau21 rule  %s  de Bruijn diagram",c);
  [drawPanel makeKeyAndOrderFront:self];
  [drawPanel setTitle: d];
  [drawTheView display];
  drawBasic=NO;
  lastDraw = 2;
return self;
}

- fullRijn:sender {char c[WNML], d[37+WNML];
    shortRijn=YES;
    wru(c);
    sprintf(d,"lcau21 rule  %s  de Bruijn diagram",c);
    [drawPanel makeKeyAndOrderFront:self];
    [drawPanel setTitle: d];
    [drawPanel display];
    shortRijn=NO;
    lastDraw = 1;
return self;
}

- shewRijn:sender {char d[37];
    justRijn=YES;
    sprintf(d,"lcau21 generic de Bruijn diagram");
    [drawPanel makeKeyAndOrderFront:self];
    [drawPanel setTitle: d];
    [drawPanel display];
    justRijn=NO;
    lastDraw = 3;
return self;
}

- printGUPH:sender {
    switch (lastDraw) {
      case 0: break;
      case 1: shortRijn=YES; break;
      case 2: drawBasic=YES; break;
      case 3: justRijn=YES;  break;
      default: break;}
    [drawPanel smartPrintPSCode: self];
    switch (lastDraw) {
      case 0: break;
      case 1: shortRijn=NO; break;
      case 2: drawBasic=NO; break;
      case 3: justRijn=NO;  break;
      default: break;}
return self;
}

- windowDidBecomeKey:sender {
	[rijnWindow orderFront: self];
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyGUPH:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printGUPH:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [drawTheView getBounds: &zbounds];
  [drawTheView writePSCodeInside: &zbounds to: pBoard]; 
  *flag = NO;
return self;
}

- copyGUPH:sender {
    rijnBack=NO;
    switch (lastDraw) {
      case 0: break;
      case 1: [self copyWhatsit: &shortRijn]; break;
      case 2: [self copyWhatsit: &drawBasic]; break;
      case 3: [self copyWhatsit: &justRijn];  break;
      default: break;}
    rijnBack=YES;
return self;
}

/* - - - - - - - - - - - - -    L I U L G U P H   - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new LiulGuph object.  First, an initFrame: message is sent 
 * to super to initialize LiulGuph as a View.  Next, the LiulGuph sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	[super initFrame:frameRect];
/*	[self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	crossCursor = 
		[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}

- setDelegate:anObject
/*
 * Sets the LiulGuph's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the LiulGuph's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although LiulGuph only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[WNML], ch[50], timday[30];
	int		 i=0;
  struct tm  *timeptr;
  time_t  secsnow;

       	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (rijnBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

// drawAxes(bounds.origin.x,
//	    bounds.origin.y,
//	    bounds.size.width,
//	    bounds.size.height);

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(timday,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [timeDate setStringValue: timday];

/* for the de Bruijn  diagram --- - - - - - - - - - - - - - - - - - - - - */

        if (drawBasic==YES) {
		PSsetlinewidth(3.0); 
		PSsetgray(NX_BLACK); 
   [[Font newFont: "Times-Bold" size: 48.0 matrix:NX_FLIPPEDMATRIX] set];
   for (i=0; i<KK; i++) if ([[basicStates cellAt: 0: i] state]) sijn(i,i); 
		PSstroke();
		PSsetlinewidth(0.0); 
		}

        if (justRijn==YES) {
		PSsetgray(NX_BLACK); 
		ijn(dd,KK,1); 
		PSstroke();
		}

        if (shortRijn==YES) {
  	[[Font newFont: "Courier" size: 20.0 matrix:NX_FLIPPEDMATRIX] set];
   wru(c);
   [ruleNumber setStringValue: c];
   PSmoveto(290.0,14.0);
   PSshow(c);
   PSmoveto(12.0,14.0);
  	[[Font newFont: "Courier" size: 12.0 matrix:NX_FLIPPEDMATRIX] set];
   if (cnstRijn) sprintf(ch,"evolving to %1d in %1d gen",rijnShift,rijnGen+1); 
	else {if (rijnPerm==0) 
	  sprintf(ch,"shift %1d in %1d gen",-rijnShift,rijnGen+1);
	  else 
sprintf(ch,"shift %1d in %1d gen perm %d",-rijnShift,rijnGen+1,rijnPerm);}
   PSshow(ch);
  	[[Font newFont: "Courier" size: 10.0 matrix:NX_FLIPPEDMATRIX] set];
		PSsetgray(NX_BLACK); 
		if (cnstRijn) edcst(rijnGen); else edshg(rijnGen,rijnShift);
		PSstroke();

		}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the LiulGuph is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the LiulGuph.  The LiulGuph changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the LiulGuph updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the LiulGuph, the LiulGuph 
 * registers the point and then sends a message to its delegate notifying it of 
 * the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
//	NXRect	plotRect;

         double ex, wy;
	 int    i, j, k, l, nxno, b[KK];

	[crossCursor set];

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
    
/* for the de Bruijn  diagram --- - - - - - - - - - - - - - - - - - - - - */

	if (lastDraw == 1) {
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				dbno=nearest(ex,wy);
				[nodeNumber setIntValue: dbno];

       if (theEvent->data.mouse.click==3) {} else 
       if (theEvent->data.mouse.click==2) {
         for (j=0; j<10; j++) {
	   for (i=0,l=0; i<KK; i++) {
		k=(KK*dbno+i)%dd;
		if (dbtag[k]==1) {b[l]=k; l++;}
		}
  	    if (l==0) break;
  	    k=0; if (l>1) k=((int)(random()))%l;
	    nxno=b[k];
	    dcntr[nxno][0]=dcntr[dbno][0]+30.0;
	    dcntr[nxno][1]=dcntr[dbno][1]-15.0;
	    dbno=nxno;
	    }

		} else {
	 [self lockFocus];
         PSsetinstance(YES);
	do {
 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
				PSmoveto(ex+12.0,wy);
				PSarc(ex, wy, 12.0, 0.0, 360.0);
				PSstroke();
				dcntr[dbno][0]=ex;
				dcntr[dbno][1]=wy;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {looping = NO;}	
   	} while (looping && 
	  (theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];


   shortRijn=YES;
//   [window display];
   [self display];
   shortRijn=NO;


  }
  }

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end LiulGuph.m */
