
/* 
 * ProbView.m -- Implementation file for the ProbView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "ProbView.h"
#import "drawingFuncs.h"
#import "nxlcau21.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>

int    probBack=YES;
int    drawBstein=NO;
int    drawMeanF=NO;
int    draw2Block=NO;
int    drawNBlk=NO;
int    drawEntpy=NO;
int    drawStat=NO;

#define MB 3				/* maximum binomial coefficient */

extern int  drawview;
extern char ascrule[KK][KK][KK];
extern char arule[DS], arr1[AL], arr2[AL];

extern double bx[BD];				/* bernstein factor        */
extern double bp[BD];

double blpr[KK];			/* probability from blocks	*/

void asctobin();
void asczero();
void bgraf(int n, int l);
void egraf(int n);
void gdiag(int n, int l);
void gfram(int n, int l);
void guare(int n, int l);
int  gnlsum(int m, int n);
void lifreq(int k, int l);
void moncar(int k, int l);
int  nblcur(int n);
int  nearest(double ex,double wy);
int  neisum(int m, int n);
void onegen(int n);
void twoblockfreq(int n);
void videodot(int i, int j, int l);
   
@implementation ProbView

/* - - - - - - - - - - - - -   P R O B V I E W   - - - - - - - - - - - - - */

- loadProb:sender {id prbMen;
  if (sender==rulePanel) {
       if (trivWindow!=nil && [trivWindow isVisible]) [self nxasfreq: sender];}
  else if (probWindow==nil) {
       if (![NXApp loadNibSection: "prob.nib" owner: self withNames: NO]) 
	  {return nil;}
        prbMen = [[Menu alloc] initTitle: "Probability"];

    [[prbMen 
      addItem: "Statistics" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: statWindow];

    [[prbMen 
      addItem: "Trivia" 
      action: @selector(nxasfreq:) keyEquivalent: '\0']
        setTarget: self];

    [[prbMen 
      addItem: "Bernstein" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: bernWindow];

    [[prbMen 
      addItem: "Entropy" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: entropyWindow];

    [[prbMen 
      addItem: "Mean Field" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: probWindow];

    [[prbMen 
      addItem: "2-Block" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: twoBWindow];

    [[prbMen 
      addItem: "N-Block" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: blockWindow];

    [[sender window] setSubmenu: prbMen forItem: [sender selectedCell]];
    [[sender selectedCell] performClick: sender];
    }
  return self;
  }

- printAProbWindow:sender {
    if (keyWindow==bernWindow)    [self printBern:    sender];
    if (keyWindow==blockWindow)   [self printNBlock:  sender];
    if (keyWindow==entropyWindow) [self printEntropy: sender];
    if (keyWindow==probWindow)    [self printProb:    sender];
    if (keyWindow==statWindow)    [self printStat:    sender];
    if (keyWindow==trivWindow)    [keyWindow smartPrintPSCode: sender];
    if (keyWindow==plaidWindow)   [self printPlaid:   sender];
    if (keyWindow==twoBWindow)    [self printBlock:   sender];
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyAProbView:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printAProbWindow:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyBern:sender {
NXRect zbounds;
id     pBoard;
  drawview=1;
  probBack=NO;
  drawBstein=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theBernView getBounds: &zbounds];
  [theBernView writePSCodeInside: &zbounds to: pBoard]; 
  drawBstein=NO;
  probBack=YES;
return self;
}

- copyEnpy:sender {
NXRect zbounds;
id     pBoard;
  drawview=1;
  probBack=NO;
  drawEntpy=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theEntropyView getBounds: &zbounds];
  [theEntropyView writePSCodeInside: &zbounds to: pBoard]; 
  drawEntpy=NO;
  probBack=YES;
return self;
}

- copyProb:sender {
NXRect zbounds;
id     pBoard;
  drawview=1;
  probBack=NO;
  drawMeanF=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theProbView getBounds: &zbounds];
  [theProbView writePSCodeInside: &zbounds to: pBoard]; 
  drawMeanF=NO;
  probBack=YES;
return self;
}

- copyStat:sender {
NXRect zbounds;
id     pBoard;
  drawview=5;
  probBack=NO;
  drawStat=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theStatView getBounds: &zbounds];
  [theStatView writePSCodeInside: &zbounds to: pBoard]; 
  drawStat=NO;
  probBack=YES;
return self;
}

- copyTwoB:sender {
NXRect zbounds;
id     pBoard;
  drawview=1;
  draw2Block=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theTwoBView getBounds: &zbounds];
  [theTwoBView writePSCodeInside: &zbounds to: pBoard]; 
  draw2Block=NO;
return self;
}

- copyNBlock:sender {
NXRect zbounds;
id     pBoard;
  drawview=4;
  drawNBlk=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theBlockView getBounds: &zbounds];
  [theBlockView writePSCodeInside: &zbounds to: pBoard]; 
  drawNBlk=NO;
return self;
}

- copyAProbView:sender {
    if (keyWindow==bernWindow)    [self copyBern:   sender];
    if (keyWindow==blockWindow)   [self copyNBlock: sender];
    if (keyWindow==entropyWindow) [self copyEnpy:   sender];
    if (keyWindow==probWindow)    [self copyProb:   sender];
    if (keyWindow==plaidWindow)   [self copyPlaid:  sender];
    if (keyWindow==statWindow)    [self copyStat:   sender];
    if (keyWindow==twoBWindow)    [self copyTwoB:   sender];
return self;
}

- drawEntropy:sender {
    drawview=1;
    drawEntpy=YES;
    [self display];
    drawEntpy=NO;
return self;
}

- printEntropy:sender {
   drawview=1;
   drawEntpy=YES;
   [keyWindow printPSCode:self];
   drawEntpy=NO;
return self;
}


- drawBern:sender {
    drawview=1;
    drawBstein=YES;
    [self display];
    drawBstein=NO;
    [window setDocEdited:YES];
return self;
}

- printBern:sender {
   drawview=1;
   drawBstein=YES;
   [keyWindow printPSCode:self];
   drawBstein=NO;
return self;
}

- enterBern:sender {
      asczero();
	if ([[bernMatrix cellAt: 2: 0] state]) ascrule[0][0][0]='1';
	if ([[bernMatrix cellAt: 0: 1] state]) ascrule[1][0][0]='1';
	if ([[bernMatrix cellAt: 1: 1] state]) ascrule[0][1][0]='1';
	if ([[bernMatrix cellAt: 2: 1] state]) ascrule[0][0][1]='1';
	if ([[bernMatrix cellAt: 0: 2] state]) ascrule[1][1][0]='1';
	if ([[bernMatrix cellAt: 1: 2] state]) ascrule[1][0][1]='1';
	if ([[bernMatrix cellAt: 2: 2] state]) ascrule[0][1][1]='1';
	if ([[bernMatrix cellAt: 2: 3] state]) ascrule[1][1][1]='1';
     [rulePanel setAllRules: self];
     [bernWindow setDocEdited: NO];
return self;
}

- resetBern:sender {int i, j; 
    for (i=0; i<TS; i++) for (j=0; j<MB; j++) {
	[[bernMatrix cellAt: j: i] setIntValue: NO];
	}
}

- copyRule:sender {
      [self resetBern: self];
	[[bernMatrix cellAt: 2: 0] setIntValue: ascrule[0][0][0]-'0'];
	[[bernMatrix cellAt: 0: 1] setIntValue: ascrule[1][0][0]-'0'];
	[[bernMatrix cellAt: 1: 1] setIntValue: ascrule[0][1][0]-'0'];
	[[bernMatrix cellAt: 2: 1] setIntValue: ascrule[0][0][1]-'0'];
	[[bernMatrix cellAt: 0: 2] setIntValue: ascrule[1][1][0]-'0'];
	[[bernMatrix cellAt: 1: 2] setIntValue: ascrule[1][0][1]-'0'];
	[[bernMatrix cellAt: 2: 2] setIntValue: ascrule[0][1][1]-'0'];
	[[bernMatrix cellAt: 2: 3] setIntValue: ascrule[1][1][1]-'0'];
     [self drawBern: self];
     [window setDocEdited: NO];
return self;
}

/* -------------------------------- L C A U -------------------------------- */

/* -------------------------------- R I J N -------------------------------- */

/* -------------------------------- P R O B -------------------------------- */

- drawProb:sender {
    drawview=1;
    drawMeanF=YES;
    [self display];
    drawMeanF=NO;
return self;
}

- printProb:sender {
   drawview=1;
   drawMeanF=YES;
   [keyWindow printPSCode:self];
   drawMeanF=NO;
return self;
}

- drawBlock:sender {
    drawview=1;
    draw2Block=YES;
    [self display];
    draw2Block=NO;
return self;
}

- printBlock:sender {
   drawview=1;
   draw2Block=YES;
   [keyWindow printPSCode:self];
   draw2Block=NO;
return self;
}

- drawNBlock:sender {
    drawview=4;
    drawNBlk=YES;
    [self display];
    drawNBlk=NO;
return self;
}

- printNBlock:sender {
   drawview=4;
   drawNBlk=YES;
   [keyWindow smartPrintPSCode:self];
   drawNBlk=NO;
return self;
}

/* - - - - - - - - - -  S T A T I S T I C S   W I N D O W  - - - - - - - - - */

- drawStat:sender {
    drawview=5;
    drawStat=YES;
    [self display];
    drawStat=NO;
return self;
}

- printStat:sender {
   drawview=5;
   drawStat=YES;
   [keyWindow smartPrintPSCode:self];
   drawStat=NO;
return self;
}

- nxmoncar:sender {
int i, j, k, n, b[KK], bb[KK][KK];
double bf[KK], bbf[KK][KK];
double a[KK], aa[KK], d[KK], el, en, t, tt, s;

n=[[statLines selectedCell] tag];
printf("n : %d\n", n);
asctobin();
el=1.0/((double)AL);
en=1.0/((double)n);
for (i=0; i<KK; i++) {a[i]=0.0; aa[i]=0.0;}
for (k=0; k<n; k++) {
  onegen(AL);
  for (i=0; i<KK; i++) b[i]=0;
  for (i=0; i<AL; i++) b[arr1[i]]++;
  for (i=0; i<KK; i++) bf[i]=((double)(b[i]))*el;
  for (i=0; i<KK; i++) {a[i]+=bf[i]; aa[i]+=bf[i]*bf[i];}
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) bb[i][j]=0;
  for (i=1; i<AL; i++) bb[arr1[i-1]][arr1[i]]++;
  bb[arr1[AL-1]][arr1[0]]++; 
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
    bbf[i][j]=((double)(bb[i][j]))*el;
  videodot((int)(100.0*bbf[1][1]),(int)(100.0*bf[1]),3);
  }
for (i=0; i<KK; i++) {a[i]*=en; aa[i]*=en;}
s=1.0/log((double)KK);
t=0.0; tt=0.0;
for (i=0; i<KK; i++) t+=bf[i]*log(bf[i]);
for (i=0; i<KK; i++) d[i]=sqrt(aa[i]-a[i]*a[i]);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) tt+=bbf[i][j]*log(bbf[i][j]);
      [statLamb setDoubleValue: 1.0-bf[0]];
      [statEntro setDoubleValue: -t*s];
      [statEnpai setDoubleValue: -0.5*tt*s];
 printf("average      = %5.3f, %5.3f\n",a[0],a[1]);
 printf("deviation    = %5.3f, %5.3f\n",d[0],d[1]);
      [statSDevi setDoubleValue: d[1]];
  for (i=0; i<KK; i++) [[statState cellAt: 0: i] setDoubleValue: bf[i]];
  for (i=0; i<KK; i++)  for (j=0; j<KK; j++) {
	[[statPair cellAt: i: j] setDoubleValue: bbf[i][j]];
	[[statProd cellAt: i: j] setDoubleValue: bf[i]*bf[j]];
    }
 return self;
}

/* - - - - - - - - - - - -  T R I V I A   P A N E L  - - - - - - - - - - - - */

/* graph the frequencies of ascrule 		 */
/* put dot at coordinates (u,v)			 */
/* percentages evolving into each value		 */
/* percentages shifting - adds to more than 100% */
/* entropy and Langton's lambda = % nonquiescent */

- nxasfreq:sender {
int    i, j, i0, i1, i2, i3, k;
int    stat[KK], statp[KK][KK];		/* statistic counts */
int    shif[RR];			/* shift counts */
int    xe;				/* xenophobic count */
double staf[KK], stapf[KK][KK], t, pp, qq;

xe=0;
pp=1.0/((double)(DS));
qq= pp/((double)KK);
for (i=0; i<RR; i++) shif[i]=0;
for (i=0; i<KK; i++) {stat[i]=0; for (j=0; j<KK; j++) statp[i][j]=0; }
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++) {
  j=ascrule[i0][i1][i2]-'0';
  stat[j]++;
  if (j==i0) shif[0]++;
  if (j==i1) shif[1]++;
  if (j==i2) shif[2]++;
  if (j==i0 || j==i1 || j==i2) xe++;
    for (i3=0; i3<KK; i3++) {
      k=ascrule[i1][i2][i3]-'0';
      statp[j][k]++;
     }
  }
[trivWindow orderFront: sender];
for (i=0; i<KK; i++) staf[i]=((double)stat[i])*pp;
for (i=0; i<KK; i++) [[trivState cellAt: 0: i] setDoubleValue: staf[i]];
for (i=0; i<RR; i++)	
        [[trivIndx cellAt: i: 0] setDoubleValue: ((double) shif[i])*pp];
for (i=0, t=0.0; i<KK; i++) t+=staf[i]*log(staf[i]);
	[trivLamb setDoubleValue: 1.0-staf[0]];
	[trivEntro setDoubleValue: -t/log((double)KK)];
for (i=0, t=0.0; i<KK; i++) for (j=0; j<KK; j++) {
        stapf[i][j]= ((double) statp[i][j])*qq;
	[[trivPairs cellAt: i: j] setDoubleValue: stapf[i][j]];
  	[[trivProds cellAt: i: j] setDoubleValue: staf[i]*staf[j]];
	t+=stapf[i][j]*log(stapf[i][j]);
	}
[trivPEntro  setDoubleValue: -t/log((double)(KK*KK))];
// [trivXeno  setDoubleValue: ((double)(DS-xe))*pp at: 0];
// [trivXeno  setDoubleValue: ((double)xe)*pp at: 1];
return self;
}

/* --------------------------   P r o b    V i e w   ----------------------- */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new ProbView object.  First, an initFrame: message is sent 
 * to super to initialize ProbView as a View.  Next, the ProbView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];
/*	[self translate: 
		floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	crossCursor = 
		[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];


  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}

- setDelegate:anObject
/*
 * Sets the ProbView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}


- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the ProbView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although ProbView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[WNML], ch[40];
	int		 i, j, k=0, n;
        double           p, q, s, dp, r;
	NXPoint		 *aPoint;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (probBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawBstein==YES) {
	  for (i=0; i<BD; i++) bp[i]=0.0;
	  for (i=0; i<TS; i++)
	  for (j=0; j<MB; j++) {
		if ([[bernMatrix cellAt: j: i] state]) bp[i]+=1.0;
		}
	  gfram(100,3);
	  gdiag(100,3);
	  bgraf(2,100);
	  }

	if (drawEntpy==YES) {egraf(100);}

        if (drawMeanF==YES) {
	sscanf([[probHowMany selectedCell] title],"%d",&k);
	lifreq(k,3);
	for (i=0; i<6; i++) if ([[probEI cellAt:i:0] state]) procur('e',5-i);
	for (i=0; i<6; i++) if ([[probEI cellAt:i:1] state]) procur('i',5-i);
	}

	if (drawStat==YES) {
	    if (strcmp([[statNhood selectedCell] title],"totalistic")==0) {
		[self nxmoncar: self];
		printf("totalistic\n");
		neisum(AL,48);
                }
	    if (strcmp([[statNhood selectedCell] title],"general")==0) {
		[self nxmoncar: self];
		printf("general\n");
		gnlsum(AL,48);
                }
	  }

	if (draw2Block==YES) {
	if ([tblokCtur state]) twoblockfreq(100);
	if ([tblokDiag state]) gdiag(100,2);
	if ([tblokGrid state]) gfram(100,2);
	if ([tblokPara state]) guare(100,2);
	sscanf([[tblokHowMany selectedCell] title],"%d",&k);
	moncar(k,2);
	}

	if (drawNBlk==YES) {
	  nblcur([[nblokLen selectedCell] title][0]);
	  for (i=0; i<KK; i++) 
		[[nblokAve cellAt: 0: i] setDoubleValue: blpr[i]];
	  }

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the ProbView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo:width :height];
	if (drawview==4||drawview==7||drawview==10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else 
		[self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the ProbView.  The ProbView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events. As it receives mouse-dragged events, the ProbView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the ProbView, the ProbView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect, cellRect;

         double  ex, wy;
	 int k;

	[crossCursor set];
   	[self getBounds:&plotRect];
 	NXSetRect(
		&cellRect, plotRect.origin.x, plotRect.origin.y, 100.0, 20.0);

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
//                ex=aPoint.x;
//                wy=aPoint.y;
				PSsetgray(NX_BLACK);
//				PSmoveto(ex+12.0,wy);
//				PSarc(ex, wy, 12.0, 0.0, 360.0);
//				PSstroke();
//				cntr[k][0]=ex;
//				cntr[k][1]=wy;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {looping = NO;}	
   	} while (looping && (theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];


	}
	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}


/* end ProbView.m */
