
/* 
 * TripleView.m -- Implementation file for the TripleView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "TripleView.h"
#import "drawingFuncs.h"
#import "nxlcau21.h"
#import "nxardn21.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>
#include <stdio.h>
#include <time.h>

int    filltrback=YES;

extern parneih    amat;
extern parneih    bmat;

thrparn    stmat;
thrparn    ttmat;
thrparn    utmat;
thrparn    vtmat;

int    trno=0;			/* node number in triple diagram	*/
int    trnd;			/* minimum  triple separation		*/
int    trxd;			/* maximum triple separation		*/
int    trnl;			/* minimum loop length			*/
int    trxl;			/* maximum loop length			*/
int    errgset[TDN];		/* members of the ergodic set		*/
double tcntr[TDN][2];		/* triple diagrm coordinates		*/

int    antitr(int n);
void   asctobin();
int    closetotr(double x, double y);
void   edTriple(int l, int pc, int pa, int pl, int pv, int op);
void   findersettr(int *e, thrparn *t);
void   tripleflip();
void   howdowntr(int *sud);
int    leveltr(int n);
int    tmax(int i, int j, int k);
int    tmid(int i, int j, int k);
int    tmin(int i, int j, int k);
int    trnins(int i, thrparn *z);
int    trouts(int i, thrparn *z);
void   trxtrmd(int *min, int *max);
void   trxtrml(int *min, int *max);
int    putleveltr(thrparn *zmat, thrparn *xmat, int n);
int    shflevtr(int g, int s);
void   suanctr();
void   uptheretr(int *sud);

@implementation TripleView


/* ----------------------------- T R I P L E ----------------------------- */

/* put up the principal window for the TripleView. Since triple.nib is a   */
/* secondary .nib file, it is not loaded until there is a demand, which    */
/* would usually arise from the "Triple" option in the main menu.	   */

- loadTriple:sender {
  if (tripleWindow==nil) {
    if (![NXApp loadNibSection: "triple.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  [tripleWindow makeKeyAndOrderFront: nil];
  return self;
  }

- windowDidBecomeKey:sender {
	[menuCopyV setTarget: self];
	[menuCopyV setAction: @selector(copyTripleView:)];
	[menuCopyV setEnabled: YES];
	[menuCopyW setTarget: self];
	[menuCopyW setAction: @selector(copyTripleWindow:)];
	[menuCopyW setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printTriple:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender
{
	[menuCopyV setEnabled: NO];
	[menuCopyW setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- drawTriple:sender {
int    i, j, k;
int    i0, i1;
int    j0, j1;
asctobin();
zmat(&amat);   djmat(&amat,0);
zmat(&bmat);   djmat(&bmat,1);
zzzmat(&ttmat); tencu(&ttmat,&amat,&bmat);
stmat=utmat=ttmat; 
  i=2; do {
    mmmmat(&vtmat,&utmat,&ttmat);
    k=putleveltr(&stmat,&vtmat,i);
    utmat=vtmat;
    i++;
printf("running level %3d\n",k);
    } while (k!=0);
//  for (i=0; i<DN; i++) for (j=0; j<DN; j++) 
//	[[trTriv2 cellAt: i: j] setIntValue: stmat.two[i][j]];
  trxtrmd(&trnd,&trxd);
  [trTriv1 setIntValue: trnd at: 2];
  [trTriv1 setIntValue: trxd at: 3];
  trxtrml(&trnl,&trxl);
  [trTriv1 setIntValue: trnl at: 0];
  [trTriv1 setIntValue: trxl at: 1];

  utmat=ttmat;
  for (i=0; i<TDN; i++) errgset[i]=1;
  for (i=0; i<5; i++) findersettr(errgset,&utmat);

printf("errgset found\n");

  for (i=0; i<DN; i++) for (j=0; j<DN; j++) for (k=0; k<DN; k++) {
    tcntr[i+DN*(j+DN*k)][0]=(double)( 25+100*i+30*(DN-k));
    tcntr[i+DN*(j+DN*k)][1]=(double)(150+100*j-30*(DN-k));
    }
printf("drawTriple finished\n");
  [self redrawTriple: self];
return self;
}

- redrawTriple:sender {
printf("redrawTriple\n");
  [nodeNumber setIntValue: trno];
  if ([ordTriple state]) tripleflip();
  [self display];
return self;
}

- printTriple:sender {[tripleWindow smartPrintPSCode:self]; return self;}

- copyTripleView:sender {
NXRect zbounds;
id     pBoard;
  filltrback=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theTripleView getBounds: &zbounds];
  [theTripleView writePSCodeInside: &zbounds to: pBoard]; 
  filltrback=YES;
return self;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

void trigrid () {
int i, j, k;
  for (i=0; i<DN; i++) for (j=0; j<DN; j++) {
    PSmoveto((double)( 25+100*i+30*DN),(double)(150+100*j-30*DN));
    PSlineto((double)( 25+100*i+30   ),(double)(150+100*j-30   ));
    }
  for (j=0; j<DN; j++) for (k=0; k<DN; k++) {
    PSmoveto((double)( 25           +30*(DN-k)),(double)(150+100*j-30*(DN-k)));
    PSlineto((double)( 25+100*(DN-1)+30*(DN-k)),(double)(150+100*j-30*(DN-k)));
    }
  for (i=0; i<DN; i++) for (k=0; k<DN; k++) {
    PSmoveto((double)( 25+100*i+30*(DN-k)),(double)(150           -30*(DN-k)));
    PSlineto((double)( 25+100*i+30*(DN-k)),(double)(150+100*(DN-1)-30*(DN-k)));
    }
}

int putleveltr(zmat,xmat,n) thrparn *zmat, *xmat; int n; {int i, k;
k=0;
for (i=0; i<TDN*TDN; i++) 
  if (zmat->one[i]==0 && xmat->one[i]!=0) {zmat->one[i]=n; k=1;}
return k;
}

void edTriple(l,pc,pa,pl,pv,op) int l, pc, pa, pl, pv, op; {
int    i0, i1, i2;
int    ii0, ii1, ii2;
int    i, j, m;
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, t;

  m=0;
  r=10.3;
  for (i=0; i<TDN; i++) {
     			if (pv && errgset[i]==0) continue; 
    for (j=0; j<TDN; j++) {
     			if (pv && errgset[j]==0) continue; 
	if (ttmat.two[i][j]==0) continue;
	m++;
	ex=tcntr[i][0];
	wy=tcntr[i][1];
	PSmoveto(ex,wy);
	x=tcntr[j][0];
	y=tcntr[j][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	eks=x-r*cos(t);
	wye=y-r*sin(t);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSmoveto(x+10.3,y);
	PSarc(x, y, 10.3, 0.0, 360.0);
	PSmoveto(ex+10.3,wy);
	PSarc(ex, wy, 10.3, 0.0, 360.0);
	}

   if (pl==YES) {
   ex=tcntr[i][0];
   wy=tcntr[i][1];
   PSmoveto(ex-10.0,wy+5.0);
	i0=i%K2;
	i1=(i/K2)%K2;
	i2=i/K4;
	ii0=tmin(i0,i1,i2);
	ii1=tmid(i0,i1,i2);
	ii2=tmax(i0,i1,i2);
	if (op) sprintf(s,"(%1d,%1d,%1d)",ii0,ii1,ii2);
	   else sprintf(s,"(%1d,%1d,%1d)",i0,i1,i2);
   PSshow(s);
   }
  }
	printf("total links = %5d\n",m);
}

/* find index of nearest center to x, y (cycles) */
int neartotr(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<TDN; i++) {
  ex=x-tcntr[i][0];
  wy=y-tcntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* find node number of the reflected triple */
int antitr(n) int n; {return (n/KK)+KK*(n%KK);}

/* min, max of three integers */ 
int tmin(i,j,k) int i, j, k; {return i<j?(i<k?i:k):(j<k?j:k);}
int tmid(i,j,k) int i, j, k; {return i<j?(i>k?i:k):(j>k?j:k);}
int tmax(i,j,k) int i, j, k; {return i>j?(i>k?i:k):(j>k?j:k);}

/* flip the location of the disordered members of symmetric triples */ 
void tripleflip() {int i, j, k, ii, jj, kk;
  for (i=1; i<DN; i++) for (j=0; j<DN; j++) for (k=0; k<DN; k++) {
    ii=tmin(i,j,k);
    jj=tmid(i,j,k);
    kk=tmax(i,j,k);
    tcntr[i+DN*(j+DN*k)][0]=tcntr[ii+DN*(jj+DN*kk)][0];
    tcntr[i+DN*(j+DN*k)][1]=tcntr[ii+DN*(jj+DN*kk)][1];
    }
}

/* calculate number of incoming links for triple diagram nodes */
int trnins(i,z) int i; thrparn *z; {int j, k;
for (k=0, j=0; j<TDN; j++) if (z->two[j][i]!=0) k++;
return k;
}

/* calculate number of incoming links for triple diagram nodes */
int trouts(i,z) int i; thrparn *z; {int j, k;
for (k=0, j=0; j<TDN; j++) if (z->two[i][j]!=0) k++;
return k;
}

/* if there are no ins to i, make link(i,j)=0 for all j */
void findersettr(e,t) int *e; thrparn *t; {int i, j; thrparn z;
z=*t;
for (i=0; i<TDN; i++) if (trnins(i,&z)==0) {
  e[i]=0;
  for (j=0; j<TDN; j++) z.two[i][j]=0;
  }
for (i=0; i<TDN; i++) if (trouts(i,&z)==0) {
  e[i]=0;
  for (j=0; j<TDN; j++) z.two[j][i]=0;
  }
*t=z;
}

/* find extremal distances in the triple distance matrix */
void trxtrmd(min, max) int *min, *max; {int i, n, x, s;
x=0; n=TDN*TDN;
for (i=0; i<TDN*TDN; i++) {s=stmat.one[i]; if (s<n) n=s; if (s>x) x=s;}
*min=n; *max=x;
}

/* find extremal loops in the triple distance matrix */
void trxtrml(min, max) int *min, *max; {int i, n, x, s;
x=0; n=TDN*TDN;
for (i=0; i<TDN; i++) {s=stmat.two[i][i]; if (s<n) n=s; if (s>x) x=s;}
*min=n; *max=x;
}

/* - - - - - - - - - - - - -    P A I R V I E W   - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new TripleView object.  First, an initFrame: message is sent 
 * to super to initialize TripleView as a View.  Next, the TripleView sets its 
 * own state -- that it is opaque and that the origin of its coordinate system 
 * lies in the center of its area.  It then creates and initializes its 
 * associated objects, a Storage object, an NXCursor, and a Cell.  Finally, it 
 * loads into the Window Server some PostScript procedures that it will use in 
 * drawing itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];
/*	[self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	crossCursor = [NXCursor newFromImage:
		[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];
	loadPSProcedures();

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}

- setDelegate:anObject
/*
 * Sets the TripleView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the TripleView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although TripleView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[30], timday[30];
	unsigned int	 i, j, k;
        double           p, q, s, dp;
  struct tm  *timeptr;
  time_t  secsnow;

int     i0, i1, ss;
	
	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (filltrback) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(timday,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);

/* for the triple diagram -- - - - - - - - - - - - - - - - - - - - - - - - - */

  	[[Font newFont: "Courier" size: 10.0 matrix:NX_FLIPPEDMATRIX] set];
   wru(c);
   [ruleNumber setStringValue: c];

   [timeDate setStringValue: timday];

		PSsetlinewidth(0.0);
		PSsetgray(NX_LTGRAY); 
		trigrid();
		PSstroke();
		PSsetgray(NX_BLACK); 
		edTriple(0,YES,YES,YES,[treeShow state],[ordTriple state]);
		PSstroke();
   [timeDate setStringValue: timday];
		PSstroke();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the TripleView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the TripleView.  The TripleView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the TripleView 
 * updates the readOut text Cell with the cursor's coordinates.  If the user 
 * releases the mouse button while the cursor is over the TripleView, the 
 * TripleView registers the point and then sends a message to its delegate 
 * notifying it of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;

         double  x, y, ex, wy, eks, wye;
	 int     i, j, k;

	[crossCursor set];

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];

/* for the triple diagram  - - - - - - - - - - - - - - - - - - - - - - - - - */

 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				trno=neartotr(ex,wy);
				[nodeNumber setIntValue: trno];

       if (theEvent->data.mouse.click==3) {} else 
       if (theEvent->data.mouse.click==2) {} else {
	[self lockFocus];
         PSsetinstance(YES);
	do {
 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
				PSmoveto(ex+12.0,wy);
				PSarc(ex, wy, 12.0, 0.0, 360.0);
				PSstroke();
				tcntr[trno][0]=ex;
				tcntr[trno][1]=wy;
				if ([ordTriple state]) {
					k=antitr(trno);
					tcntr[k][0]=ex;
					tcntr[k][1]=wy;
					}
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	    	/* on mouse-up, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent=[NXApp getNextEvent:
		NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

   [self display];
	}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end TripleView.m */
