
/* nxlcau21.c				*/
/* (2,1) Linear Cellular Automaton	*/

/* Reference:				*/
/*					*/
/*    Kenneth E. Perry			*/
/*    Abstract Mathematical Art		*/
/*    BYTE				*/
/*    December, 1986			*/
/*    pages 181-192			*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1990		*/
/*    Copyright (C) 1992		*/
/*    Copyright (C) 1994		*/
/*    Harold V. McIntosh		*/
/*    Gerardo Cisneros S.		*/

/* G. Cisneros, 4.3.87						*/
/* 10 April 1987 - modified for (4,2) [HVM]			*/
/* 26 April 1987 - Multiple menus [HVM]				*/
/* 28 April 1987 - back modified to (4,1) [HVM]			*/
/* 28 April 1987 - version for XVI Feria de Puebla [HVM]	*/
/* 14 May 1987 - modified for (3,1) and general rule [HVM]	*/
/* 19 May 1987 - modified for (2,2) [HVM]			*/
/* 20 May 1987 - modified for (2,3) [HVM]			*/
/* 21 May 1987 - Wolfram rule number [HVM]			*/
/* 8 June 1987 - general rule for (4,1) [HVM]		 	*/
/* 12 June 1987 - cartesian product of (2,1) rules [HVM]	*/
/* 12 June 1987 - (2,1) rule with memory  [HVM]			*/
/* 14 June 1987 - individual cycles of evolution [HVM]		*/
/* 17 June 1987 - p-adic representation in plane [HVM]		*/
/* 22 June 1987 - 2 speed gliders via 16 transitions [HVM]	*/
/* 26 June 1987 - push, pop the rule [HVM]			*/
/* 26 June 1987 - conserve position of rule cursor [HVM]	*/
/* 27 June 1987 - incremental rule construction [HVM]		*/
/* 29 June 1987 - conserve position of cell pointer [HVM]	*/
/* 30 June 1987 - mark & unmark transitions, xchg x&X [HVM]	*/
/* 25 July 1987 - display and edit de Bruijn diagrams [HVM]	*/
/* 27 July 1987 - graph of transition probabilities [HVM]	*/
/* 4 September 1987 - PROB21.C for option 't' [HVM]		*/
/* 21 October 1987 - program disks disappeared			*/
/* 20 December 1987 - program reconstructed from listings	*/
/* 20 February 1988 - RIJN21.C for option 'd' [HVM]		*/
/* 11 March 1989 - x option in trule [HVM]			*/
/* 16 March 1989 - *+- option to change screen color [HVM]	*/
/* 22 December 1992 - version for NeXT [HVM]			*/

/* # include <bdos.h>*/
# include <math.h>
# include "keybrd.h"
# include "nxlcau21.h"
# include "rec.h"

# define DROWS		20	/* rows in demo menu	 */
# define DCOLS		32	/* second col in demo m. */

# define NX		 8	/* number of sample rules */

extern printf();
extern sprinf();
extern scanf();
extern random();
void serms(double x, double y, int l, int m);
void nxlcau21();
int  dmenu();
void edrule();
void edtrule();
void edline();
void evolve(int sc);
void asctobin();
void onegen(int j);
void ranlin(double p);
void tuto();
void rmenu();
void tmenu();
void lmenu();
void wmenu(int n);
void xmenu(int n);
void xblnk();
void xtoasc(char a[DS], int n);
void totoasc();
void nutoto(int x);
int  totonu(int i);
int  istot();
int  lim(int i, int j, int k);
void scrrul();
void wolnum(int ii[KK][KK][KK],char c);
void woruno(int i, int j);
void tscrrul();
void scrstr(char *s);
int  kbdst();
char kbdin();
int  numin(int n);
void rule22();
void savearry();
void renewarry();
void reflasc();
void compasc();

void edtri();
void edard();
void edijn();

void videoputc(int c, int i);
void videocursor(int p, int i, int j);
void videodot(int j, int i, int l);

void videomode(int i);
void videoscroll(int ul, int ur, int ll, int lr, int d, int i);
void videopalette(int i);
void videocattr(int i, int j, int k, int l);

//void ranlin_p();
void xtoasc_p(int n);

char xrule[NX][KK][KK][KK];

char ixrule[NX][DS]= {

	"01001000",	/* 1 - rule 18        	*/
	"01011010",	/* 2 - rule 90		*/
	"01101000",	/* 3 - rule 22       	*/
	"01101001",	/* 4 - rule 150         */
	"01111000",	/* 5 - rule 30		*/
	"10010010",	/* 6 - rule 73		*/
	"10100110",	/* 7 - rule 101		*/
	"11111101"	/* 8 - rule 191        	*/

	};
char   recrule[NY][CLEN]={
        
        "(\"18\" W;  [ 1 - rule 01001000 ])",
	"(\"90\" W;  [ 2 - rule 01011010 ])",
	"(\"22\" W;  [ 3 - rule 01101000 ])",
	"(\"150\" W; [ 4 - rule 01101001 ])",
	"(\"30\" W;  [ 5 - rule 01111000 ])",
	"(\"73\" W;  [ 6 - rule 10010010 ])",
	"(\"101\" W; [ 7 - rule 10100110 ])",
	"(\"191\" W; [ 8 - rule 11111101 ])"

	};
char  reverdemo[4][CLEN]= {
        
        "(\"240\" W;  [ rever : 170   (1)/1 ])",
	"(\"170\" W;  [ rever : 240   (2)/3 ])",
	"(\"85\"  W;  [ rever : 15   (5)/2 ])", 
	"(\"15\"  W;  [ rever : 85   (6)/3 ])"
     };

   
char   gf2rule[13];			/* coefficients of GF2 rule	*/
char   tarule[TS];
char   ttrule[TS];			/* duda en colocacion */
char   strule[KK][TS-1];
char   ascrule[KK][KK][KK];
char   auxrule[KK][KK][KK];
int    binrule[KK][KK][KK];
char   arr1[AL], arr2[AL], arr3[AL];
int    di, dj;				/* demo pointer 		*/
int    ix0, iy0;			/* origin for pen moves 	*/
double wmul[KK], wmvl[KK];		/* left mass point 		*/
double wmur[KK], wmvr[KK];		/* right mass point 		*/
double bp[BD];				/* Bernstein coefficients 	*/
double rprob;				/* probability of live cell (line) */
int    cc, dd=4;			/* # points in de Bruijn ring	*/
int    mc, nc, nl;			/* column & line control	*/


parneih    amat;
parneih    bmat;
parneih    fmat;
parneih    gmat;
parneih    hmat;
parneih    xmat;

twoparn    smat;
twoparn    tmat;
twoparn    umat;
twoparn    vmat;


Inst xprg[10][XLEN];
Inst rprg[PLEN];
char cstr[CLEN];
char idef[1][30]= { " (()) " };  //(z(!100!0r:;)1r2r(0r:;);) a" };


double blpr[KK];			/* probability from blocks	*/


void nxlcau21() {
int  i, j, i0, i1, i2;
int  more='r', sc=0;
char a, b, c;

for (i=0; i<TS; i++) ttrule[i]='0';


    videopalette(WHCYMAG);				/* white/cyan/magenta */

    tuto();
    while (!kbdst()) rand();				/* wait for keypress */
    kbdin();						/* ignore it */
    videomode(T80X25);
    videoscroll(3,0,5,71,0,3);				/* menu on blue background */
    videoscroll(19,0,24,71,0,3);
    i=(rand()%NX);
    di=i%DROWS;
    dj=(i/DROWS==1?DCOLS:1);
    xtoasc(ascrule,i);
//     rule[DS]=0;
    ranlin(0.5);					/* random initial array */

    while (more!='n') {					/* execute multiple runs */
    rmenu();
    lmenu();
    while (0<1) {					/* set up one run */
    c=kbdin();
    if (c=='g') break;					/* go draw graph */
    if (c=='q') more='n';				/* quit for good */
    if (more=='n') break;
    switch (c) {
	case '*': sc=2; break;				/* white on red */
	case '+': sc=1; break;				/* white screen */
	case '-': sc=0; break;				/* blue screen */
	case '@':					/* numbered tot rule */
	    nutoto(numin(0));
	    totoasc();
	    rmenu();
	    videocursor(0,4,0);
	    break;
	case '$':					/* dozen totalistics */
	    j=numin(0);
	    for (i=0; i<12; i++) {
	      nutoto(j+i);
	      totoasc();
	      ranlin(0.5);
	      evolve(sc);
	      }
	    videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
	case 'T':					/* totalistic rule */
	    xblnk();
	    tmenu();
	    edtrule();
	    totoasc();
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	      ascrule[i0][i1][i2]=ttrule[i0+i1+i2];
	    videocursor(0,4,0);
	    rmenu();
	    xmenu(totonu(0));
	    break;
	case 't':					/* probability calculation */
	    edtri();
	    rmenu();
	    lmenu();
	    break;
        case 'r':					/* edit rule */	
	    xblnk();
	    edrule();
	    videocursor(0,4,0);
	    rmenu();
	    break;
        case 'l':					/* edit cell string */
	    xblnk();
	    edline();
	    videocursor(0,3,0);
	    lmenu();
	    break;
	case '\273':					/* F1 */
        case '#':					/* read stored rule */
	    xtoasc(ascrule,lim(1,dmenu(),NX)-1);
	    rmenu();
	    lmenu();
	    break;
	case 'D':					/* run through samples */
	    for (i=0; i<NX; i++) {
	      xmenu(i);
	      xtoasc(ascrule,i);
	      ranlin(0.5);
	      evolve(sc);
	      }
	    videomode(T80X25);
	    rmenu();
	    break;
	case 'd':					/* de Bruijn diagram */
	    edijn();
	    rmenu();
	    lmenu();
	    break;
	case 'e':					/* Garden of Eden */
	    edard();
	    rmenu();
	    lmenu();
	    break;
        case 'u':					/* sparse init arry */
	    xblnk();
	    for (i=0; i<AL; i++) arr1[i]=0;
	    arr1[AL/4]=1;
            arr1[AL/2]=1;
            arr1[(3*AL)/4]=1;
            arr1[(3*AL)/4+2]=1;
	    lmenu();
            break;
	case 'w':					/* Wolfram rule # */
	    i=numin(0);
	    wmenu(i);
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	      ascrule[i0][i1][i2]='0'+i%2;
	      i/=2;
	    }}}
	    rmenu();
	    break;
	case 'x':					/* random rule */
	    xblnk();
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++) {
	      if ((KK*(KK*i0+i1)+i2)%4 == 0) i=rand();
	      ascrule[i0][i1][i2]='0'+i%2;
	      i/=2;
	    }
	    rmenu();
	    break;
	case 'y':					/* random line */
	    xblnk();
	    ranlin(0.5);
            lmenu();
	    break;
	case 'Y':					/* symmetrize rule */
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	      ascrule[i2][i1][i0]=ascrule[i0][i1][i2];      
	    break;
	case 'B':					/* begin barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[0][a-'0'][b-'0']=a;
	    ascrule[1][a-'0'][b-'0']=a;
	    ascrule[2][a-'0'][b-'0']=a;
	    rmenu();
	    break;
	case 'E':					/* end barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[a-'0'][b-'0'][0]=b;
	    ascrule[a-'0'][b-'0'][1]=b;
	    ascrule[a-'0'][b-'0'][2]=b;
	    rmenu();
	    break;
	case 'L':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][b-'0'][c-'0']=c;
	    rmenu();
	    break;
	case 'R':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][b-'0'][c-'0']=a;
	    rmenu();
	    break;
	case 'S':					/* still life link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][b-'0'][c-'0']=b;
	    rmenu();
	    break;
	case '.':					/* one cycle of evolution */
	    asctobin();
	    onegen(AL);
	    lmenu();
	    break;
	case '=':
	    for (i=1; i<8;  i++) {
	    for (j=0; j<40; j++) arr1[40*i+j]=arr1[j];}
	    lmenu();
	    break;
	case '~':
	    for (i=1; i<16;  i++) {
	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];}
	    lmenu();
	    break;
        default: break;
        }
    }
    if (more=='n') break;
    do {
    evolve(sc);
    videocursor(0,0,0);
    scrstr("More?");
    videocursor(0,0,34);
    scrstr("y/n/cr");
    more=kbdin();
    } while (more=='\015');
    videomode(T80X25);					/* reset the screen */
    if (more=='n') break;
    }
  videomode(T80X25);}    

/* list of demonstration rules */
int dmenu() {int ii, jj; char c;
videomode(T80X25);
videocursor(0,0,0); scrstr("Demonstration rules for LCAU21");
videocursor(0,1,0); scrstr(" 1 - rule 18");
videocursor(0,2,0); scrstr(" 2 - rule 90");
videocursor(0,3,0); scrstr(" 3 - rule 22 (life in 1d)");
videocursor(0,4,0); scrstr(" 4 - rule 150 (xor)");
videocursor(0,5,0); scrstr(" 5 - rule 30 (stochastic)");
videocursor(0,6,0); scrstr(" 6 - rule 73 (macrocells)");
videocursor(0,7,0); scrstr(" 7 - rule 101");
videocursor(0,8,0); scrstr(" 8 - rule 191");
videocursor(0,9,0); scrstr(" 9");
videocursor(0,10,0); scrstr("10");
videocursor(0,11,0); scrstr("11");
videocursor(0,12,0); scrstr("12");
videocursor(0,13,0); scrstr("13");
videocursor(0,14,0); scrstr("14");
videocursor(0,15,0); scrstr("15");
videocursor(0,16,0); scrstr("16");
videocursor(0,17,0); scrstr("17");
videocursor(0,18,0); scrstr("18");
videocursor(0,19,0); scrstr("19");
videocursor(0,20,0); scrstr("20");
videocursor(0,1,DCOLS); scrstr("21 ");
videocursor(0,2,DCOLS); scrstr("22 ");
videocursor(0,3,DCOLS); scrstr("23 ");
videocursor(0,4,DCOLS); scrstr("24 ");
videocursor(0,5,DCOLS); scrstr("25 ");
videocursor(0,6,DCOLS); scrstr("26 ");
videocursor(0,7,DCOLS); scrstr("27 ");
videocursor(0,8,DCOLS); scrstr("28 ");
videocursor(0,9,DCOLS); scrstr("29 ");
videocursor(0,10,DCOLS); scrstr("30 ");
videocursor(0,11,DCOLS); scrstr("31 ");
videocursor(0,12,DCOLS); scrstr("32 ");
videocursor(0,13,DCOLS); scrstr("33 ");
videocursor(0,14,DCOLS); scrstr("34 ");
videocursor(0,15,DCOLS); scrstr("35 ");
videocursor(0,16,DCOLS); scrstr("36 ");
videocursor(0,17,DCOLS); scrstr("37 ");
videocursor(0,18,DCOLS); scrstr("38 ");
videocursor(0,19,DCOLS); scrstr("39 ");
videocursor(0,20,DCOLS); scrstr("40 ");
videocursor(0,22,0); scrstr("move cursor with arrows, select with <cr>\n");
videocursor(0,23,0); scrstr("position at top with <, bottom with >.\n");
  ii=DROWS; jj=DCOLS; c='!'; /* ii = max # rows, jj = second column */
  do {
    videocursor(0,di+1,dj);
    while (!kbdst()) videocursor(0,di+1,dj);
    c=kbdin();
      switch (c) {
      case '\013': case '\310': if (di==0) di=ii-1; else di--; break; /* up arrow */
      case '\012': case '\320': if (di==ii-1) di=0; else di++; break; /* down arrow */
      case '\014': case '\315': case ' ': dj=(dj==1?jj:1); break;     /* forward */
      case '\010': case '\313': dj=(dj==1?jj:1); break;		      /* backward */
      case '<': di=0; break;					      /* home */
      case '>': di=ii-1; break;					      /* bottom */
      default: break;
      }
    } while (c!='\015');
  videomode(T80X25);
  return(di+(dj==1?0:ii)+1);
}

/* edit the rule */
void edrule() {
char c;
int  i, i0, i1, i2;

i=6; i0=0; i1=0, i2=0;
    while (0<1) {
        videocursor(0,3,i);
        c = kbdin();
        if (c == '\015') break;				/* carriage return exits */
        switch (c) {
        case '0':  case '1':				/* state */
	    ascrule[i0][i1][i2] = c;
    	    i2++;
	    if (i2>KK-1) {i2=0; i1++;}
	    if (i1>KK-1) {i1=0; i0++;}
	    if (i0>KK-1) {i2=0; }
            videocattr(0,c,3,1);
            if (i<6+DS) i++;
            break;
        case ' ': case '\315':				/* space = advance */
    	    i2++;
	    if (i2>KK-1) {i2=0; i1++;}
	    if (i1>KK-1) {i1=0; i0++;}
	    if (i0>KK-1) {i2=0; }
            if (i<6+DS) i++;
            break;
        case '\010': case '\313':			/* backspace */
	    if (i2!=0) i2--; else {i2=KK-1;
	    if (i1!=0) i1--; else {i1=KK-1;
	    if (i0!=0) i0--; else {i0=KK-1;
	    }}}
            if (i>6) i--;
            break;
	default: break;
        }
    }
}

/* edit totalistic rule */
void edtrule() {char c; int  i, k=0;
    i=0;
    while (i<TS) {
	c=ttrule[i];
	videocursor(0,3,56+i);
        videocattr(0,c,3,1);
        c = kbdin();
        if (c == '\015') break;
        switch (c) {
          case '0': case '1':				/* state */
            ttrule[i]=c;
            videocattr(0,c,5,1);
	    i++;
            break;
          case ' ': case '\315':			/* space = advance */
	    i++;
            break;
          case '\010': case '\313':			/* backspace */
            if (i!=0) i--;
	    break;
          case 'x':					/* random selection */
	    for (i=0; i<TS; i++) {
	      if (i%8==0) k=rand();
	      ttrule[i]='0'+k%KK; k/=KK;}
	    i=0;
	    tscrrul();
	    break;
	  default: break;
          }
    }
}

/* edit the line */
void edline() {
char c;
int  i, j, k, ii, jj;

    videocursor(0,19,0);
    scrstr("insert states using 0, 1,");
    videocursor(0,20,0);
    scrstr("move cursor with n(north), s(south), e(east), w(west), or");
    videocursor(0,21,0);
    scrstr("with keyboard arrows. Space, backspace move right and left.");
    videocursor(0,22,0);
    scrstr("( seeks left margin, < absolutely, { up one line, [ down one line");
    videocursor(0,23,0);
    scrstr(") seeks right margin, > absolutely, } up one line, ] down one line");
    videocursor(0,24,0);
    scrstr("carriage return exits");
    jj=4;
    ii=1;
    while (0<1) {
    ii=lim(1,ii,40);
    jj=lim(1,jj,8);
    j=jj-1;
    i=ii-1;
    videocursor(0,j+9,i);
    c=kbdin();
    if (c == '\015') {videoscroll(19,0,24,70,0,3); break;}
    switch (c) {
    case '0':  case '1':		/* enter  state */
       arr1[40*j+i]=c-'0'; ii++; break;
    case 's': case '\012': case '\320':		  jj++; break;	/* down - next line */
    case 'n': case '\013': case '\310':		  jj--; break;	/* up   - last line */
    case 'e': case '\014': case '\315': case ' ': ii++; break;	/* space = advance  */
    case 'w': case '\010': case '\313':		  ii--; break;	/* backspace */
    case '<': ii=1;  jj=1;  break;  /* absolute left */
    case '{': ii=1;  jj--;  break;  /* left one row up */
    case '(': ii=1;         break;  /* left this row */
    case '[': ii=1;  jj++;  break;  /* left next row */
    case '>': ii=40; jj=40; break;  /* absolute right */
    case '}': ii=40; jj--;  break;  /* right one row up */
    case ')': ii=40;        break;  /* right this row */
    case ']': ii=40; jj++;  break;  /* right next row */
    default: break;
        }
    videocursor(0,j+9,0);
    for (k=0; k<40; k++) videoputc('0'+arr1[40*j+k],1);
    }
}

/* display a screen of evolution	      */
/* sc = coloring scheme			      */
/* 0=black-blue, 1=black-white, 2=mauve-white */

void evolve(sc) int sc; {
int i, j;
  asctobin();
  for (j=0; j<200; j++) {		/* evolve for 200 generations */
    switch (sc) {
     case 0: for (i=0; i<AL; i++) /* 	serms((double)(2*i),(double)(2*j),arr1[i],3); */
            videodot(j,i,arr1[i]);
            break;
/*      case 1: for (i=0; i<AL; i++) videodot(j,i,arr1[i]*3); break; */
/*      case 2: for (i=0; i<AL; i++) videodot(j,i,arr1[i]+2); break; */
      default: break; }
    onegen(AL);
    }
}

/* copy ascrule over to binrul */
void asctobin() {int i, i0, i1, i2;
//    for (i=0; i<DS; i++) {arule[i] = rule[i]-'0';}
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
      binrule[i0][i1][i2]=ascrule[i0][i1][i2]-'0';
}

/* copy binrule over to ascrul */
void bintoasc() {int i, i0, i1, i2;
//    for (i=0; i<DS; i++) {arule[i] = rule[i]-'0';}
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
      ascrule[i0][i1][i2]=binrule[i0][i1][i2]+'0';
}

/* reflect ascrule */
void reflasc() {int i0, i1, i2; char t[KK][KK][KK];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
      t[i0][i1][i2]=ascrule[i2][i1][i0];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
      ascrule[i0][i1][i2]=t[i0][i1][i2];
}

/* complement ascrule */
void compasc() {int i0, i1, i2; char t[KK][KK][KK];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
      t[i0][i1][i2]=((ascrule[(i0+1)%KK][(i1+1)%KK][(i2+1)%KK]-'0')+1)%KK+'0';
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++) {
      ascrule[i0][i1][i2]=t[i0][i1][i2];
      }
}

/* evolution for one generation */
void onegen(j) int j; {int i;
  if (j<2) return;
  arr2[0]=binrule[arr1[j-1]][arr1[0]][arr1[1]];
  for (i=1; i<j-1; i++) arr2[i]=binrule[arr1[i-1]][arr1[i]][arr1[i+1]];
  arr2[j-1]=binrule[arr1[j-2]][arr1[j-1]][arr1[0]];
  for (i=0;  i<j; i++) arr1[i]=arr2[i];
}

/* calculate index, or rather the neighbors, in edline. */
 void linix(xx,i,j,nc) int xx[RR+1], i, j, nc; {int k, x;
  xx[(RR-1)/2]=x=i+nc*(j-1);
  if (i==0) xx[0]=x+nc-1; else xx[0]=x-1;
  if (i==nc-1) xx[2]=x-nc+1; else xx[2]=x+1;
  for (k=0; k<RR; k++) xx[k]=arr1[xx[k]];
  xx[RR]=arr1[x+nc];
}

/* generate a random line of cells in arr1 */
void ranlin(p) double p; {int i, c;
for (i=0; i<AL; i++) {
  c=(int)(1000.0*p);
  arr1[i]=(rand()%1000)<=c?1:0;
  }
}

/* map the automaton state into the unit square */
/*void pprob() {int i, j; float x, y, z, k;
i=AL/2; k=(float)KK; z=(float)((100/(KK*KK))*KK*KK);
x=0.0; for (j=1; j<25; j++) x=(x+(float)arr1[i+j])/k;
y=0.0; for (j=1; j<25; j++) y=(y+(float)arr1[i-j])/k;
x=(x+(float)arr1[i])/k;
videodot(100-(int)(z*(1.0-y)),1+(int)(z*x),1);
}
*/

void pprob() {int i, j; float x, y, z, k;
i=AL/2; k=(float)KK; z=(float)((128/(KK*KK))*KK*KK);
x=0.0; for (j=1; j<25; j++) x=(x+(float)arr1[i+j])/k;
y=0.0; for (j=1; j<25; j++) y=(y+(float)arr1[i-j])/k;
x=(x+(float)arr1[i])/k;
videodot(128-(int)(z*(1.0-y)),1+(int)(z*x),1);
}

/* set up a grid for the unit square */
void pgrid() {int i, j, k, l;
k=KK*KK+1;
l=195/(k-1);
for (i=0; i<k; i++) {
for (j=0; j<k; j++) {
videodot(l*i,l*j,2);
}}
}

/* tutorial and Help screen */
void tuto() {
    videomode(T80X25);
    videocursor(0,2,0);
    scrstr("<Copyright (C) 1987, 1988, 1989 - H.V.McIntosh, G.Cisneros S.>");
    videocursor(0,4,0);
    scrstr("              *** LIFE in One Dimension ***");
    videocursor(0,6,0);
    scrstr("Two States - Black(0), Cyan(1).");
    videocursor(0,8,0);
    scrstr("First neighbors - one on each side, three altogether.");
    videocursor(0,10,0);
    scrstr("Complete transition rule - random, edited, or stored.");
    videocursor(0,12,0);
    scrstr("Initial Cellular Array - random, edited, or patterned.");
    videocursor(0,14,0);
    scrstr("Submenus in options t(probabilities) and d(de Bruijn)");
    videocursor(0,15,0);
    scrstr("will be displayed in response to typing ?");
    videocursor(0,17,0);
    scrstr("Use any key to terminate a display in progress.");
    videocursor(0,21,0);
    scrstr("Now, ... press any key to continue.");
}

/* rule menu */
void rmenu() {
    videocursor(0,0,0);
    scrstr("      00001111");
    videocursor(0,1,0);
    scrstr("      00110011");
    videocursor(0,2,0);
    scrstr("      01010101");
    videocursor(0,3,0);
    scrstr("Rule: ");
    scrrul();
    videocursor(0,3,16);
    printf("  ");
    videocursor(0,3,16);
    if (istot()==1) printf("%2d",totonu(0));
    videocursor(0,5,0);
    scrstr("   r(rule), l(line), #nn(stored), g(graph), q(quit), t(prob),");
    videocursor(0,6,0);
    scrstr("   x(rand rule), y(rand line), u(unit line), d(deBruijn)");
    videocursor(0,7,0);
    scrstr("   wnn(Wolfram #), @nn(tot/rul), $nn(12 tot/rul), T(ed tot/rul).");
    videocursor(0,5,0);
    }

/* totalistic rule menu*/
void tmenu() {
    videocursor(0,2,50);
    scrstr("      0..1");
    videocursor(0,3,50);
    scrstr("rule: ");
    tscrrul();
    videocursor(0,3,56);
    }

/* line menu */
void lmenu() {int i, j;
    for (j=0; 40*j<AL; j++) {
	videocursor(0,9+j,0);
	for (i=0; i<40; i++) videoputc('0'+arr1[40*j+i],1);
	}
    videocursor(0,5,0); }

/* display Wolfram rule number */
void wmenu(n) int n;
{char xx[4]; int i, nn;  
    nn=sprintf(xx,"%3d",n);
    videocursor(0,2,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,2,40); }

/* display rule number */
void xmenu(n) int n;
{char xx[4]; int i, nn;  
    nn=sprintf(xx,"%3d",n);
    videocursor(0,1,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,1,40); }

/* clear screen space for rule numbers */
void xblnk() {
    videocursor(0,1,40);
    scrstr("     ");
    videocursor(0,2,40);
    scrstr("     ");
    videocursor(0,5,0); }

/* copy saved rule #n into active rule */
void xtoasc(a,n) char a[DS]; int n; {int i;
xmenu(n+1); for (i=0; i<DS; i++) a[i]=ixrule[n][i];}

/* change totalistic rule to general rule */
void totoasc() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
  ascrule[i0][i1][i2]=ttrule[i0+i1+i2];
}

/* change decimal totalistic rule to sum values */
void nutoto(x) int x; {int i;
for (i=0; i<TS; i++) {ttrule[i]=x%2+'0'; x/=2;}}

/* change sum values to decimal totalistic rule */
int totonu(i) int i; {int r;
  if (i<TS) r=(ttrule[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* test whether a rule is totalistic */
int istot() {char trule[TS]; int i0, i1, i2, l;
    l=1;
    trule[0]=ascrule[0][0][0];
    trule[1]=ascrule[1][0][0];
    trule[2]=ascrule[1][1][0];
    trule[3]=ascrule[1][1][1];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
     if (ascrule[i0][i1][i2]!=trule[i0+i1+i2]) l=0;      
    return l; }

/* find the totalistic rule number of a known ascii totalistic rule */
void asctotot() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
      ttrule[i0+i1+i2] = ascrule[i0][i1][i2];
}

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}

/* display the rule number on the screen */
void scrrul() {int i0, i1, i2;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++) {
  videoputc(ascrule[i0][i1][i2],1);      
  } }

/* turn ascrule into its Wolfram number and place on screen at (i,j) */
void woruno(i,j) int i, j; {
char xx[4];
int  k, l, i0, i1, i2;
  k=0;
  for (i2=KK-1; i2>=0; i2--)
  for (i1=KK-1; i1>=0; i1--)
  for (i0=KK-1; i0>=0; i0--) {
    k=2*k+(ascrule[i0][i1][i2]-'0');
    }
  k=sprintf(xx,"%3d",k);
  videocursor(0,i,j);
  for (l=0; l<k; l++) videoputc(xx[l],1);
}

/* report Wolfram rule number or variant */
void  wolnum(int ii[KK][KK][KK],char c)
 {
int  k, i0, i1, i2;
  k=0;
  switch (c) {
    case 'w':
          for (i0=0; i0<KK; i0++) 
      for (i1=0; i1<KK; i1++) 
      for (i2=0; i2<KK; i2++) 
        ii[i0][i1][i2]=(ascrule[i0][i1][i2]-'0'); 
      break;
    case 'c':
      for (i0=0; i0<KK; i0++) 
      for (i1=0; i1<KK; i1++) 
      for (i2=0; i2<KK; i2++)  
    ii[i0][i1][i2]=((ascrule[(i0+1)%KK][(i1+1)%KK][(i2+1)%KK]-'0')+1)%KK;
      break;
    case 'r':
      for (i0=0; i0<KK; i0++) 
      for (i1=0; i1<KK; i1++) 
      for (i2=0; i2<KK; i2++) 
         ii[i0][i1][i2]=(ascrule[i2][i1][i0]-'0');
      break;
    case 'b':
        for (i0=0; i0<KK; i0++) 
      for (i1=0; i1<KK; i1++) 
      for (i2=0; i2<KK; i2++) 
         ii[i0][i1][i2]=((ascrule[(i2+1)%KK][(i1+1)%KK][(i0+1)%KK]-'0')+1)%KK;
      break;
    case 'm':
      for (i0=0; i0<KK; i0++) 
      for (i1=0; i1<KK; i1++) 
      for (i2=0; i2<KK; i2++) 
         ii[i0][i1][i2]=ycc(ascrule[cyy(i2)][cyy(i1)][cyy(i0)]-'0');
      break;
   
    default: k=0; break;}
       }
/* display totalistic rule number by sum */
void tscrrul() {int i;
  for (i=0; i<TS; i++) {videoputc(ttrule[i],0); } }

/* write a string in graphics mode */
void scrstr(s) char *s;
  {for (; *s != '\0'; s++) videoputc(*s,1); }

/* keyboard status */
int kbdst() {}

/* direct keyboard input, no echo */
char kbdin() {}

/* read number from keyboard */
int numin(n) int n; {char c=0; /* var eliminada  s[2]; */
/*   c=kbdin(); */
  if (c>='0'&&c<='9') return(numin(10*n+(c-'0'))); else return(n);
}

void rule22(){
  ascrule[0][0][0]='0';
  ascrule[0][0][1]='1';
  ascrule[0][1][0]='1';
  ascrule[0][1][1]='0';
  ascrule[1][0][0]='1';
  ascrule[1][0][1]='0';
  ascrule[1][1][0]='0';
  ascrule[1][1][1]='0';
}

void rule204(){
  ascrule[0][0][0]='0';
  ascrule[0][0][1]='0';
  ascrule[0][1][0]='1';
  ascrule[0][1][1]='1';
  ascrule[1][0][0]='0';
  ascrule[1][0][1]='0';
  ascrule[1][1][0]='1';
  ascrule[1][1][1]='1';
}

void savearry() {int i; for (i=0; i<AL; i++) arr3[i]=arr1[i];}
void renewarry() {int i; for (i=0; i<AL; i++) arr1[i]=arr3[i];}

int  bdos(){return 0;}

/********* funciones agregadas a nxlcau21.c  **********/

void rule21(){
int i, j, i0, i1, i2;
for (i=0; i<NX; i++) {				/* copy to 3-index array */
i0=0; i1=0; i2=0;
for (j=0; j<DS; j++) {
  xrule[i][i0][i1][i2]=ixrule[i][j];
  i2++;
  if (i2==KK) {i2=0; i1++;}
  if (i1==KK) {i1=0; i0++;}
  if (i0==KK) {i0=0; }
  }}
    xtoasc_p((rand()%NX));
    ranlin(0.5);				/* random initial array */
}
 
/* clear ttrule to zeroes */
void ttzero() {int i; for (i=0; i<TS; i++)  ttrule[i]='0';}

/* clear strule to zeroes */
void stzero() {int i, j;
  for (i=0; i<KK; i++) for (j=0; j<SS; j++)  strule[i][j]='0';
}

/* clear gf2rule to zeroes */
void gfzero() {int i; for (i=0; i<13; i++) gf2rule[i]='0';}

/* test whether a rule is semitotalistic */
int isstot() {int i0, i1, i2, l, rule[KK][3];
    l=1;
    rule[0][0]=ascrule[0][0][0];
    rule[0][1]=ascrule[1][0][0];
    rule[0][2]=ascrule[1][0][1];
      
    rule[1][0]=ascrule[0][1][0];
    rule[1][1]=ascrule[1][1][0];
    rule[1][2]=ascrule[1][1][1];  
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
      if (ascrule[i0][i1][i2]!=rule[i1][i0+i2]) l=0;
    return l; }

/* clear auxrule to zeroes */
void auxzero() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
  auxrule[i0][i1][i2]=0;
}

/* clear ascrule to zeroes */
void asczero() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
  ascrule[i0][i1][i2]='0';
}

/* put a single i-cell on a line of j-cells */
void putionj(i,j) int i, j; 
{int k; for (k=0; k<AL; k++) arr1[k]=j; arr1[AL/2]=i;}

/* generate a random line of cells in arr1 with assigned probabilities */
void ranblin(p1,p2) double p1, p2; {
  int    i, c, i1;
  double p;
  p=p1+p2;
  i1=(int)((1000.0*p1)/p);
   for (i=0; i<AL; i++) {
    c=random()%1000;
    if (c<i1) arr1[i]=0; else arr1[i]=1;
    }
}

/* copy saved rule #n into active rule */
void xtoasc_p(n) int n; {int i0, i1, i2;
//    xmenu(n+1);
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++) ascrule[i0][i1][i2]=ixrule[n][KK*(KK*i0+i1)+i2];
}

/* change a GF2 rule to general rule */
void gftoasc() {}

/* change semitotalistic rule to general rule */
void sttoasc() {int i0, i1, i2;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
  ascrule[i0][i1][i2]=strule[i1][i0+i2];
}

/* end nxlcau21.c */
