
/* nxprob21.c							*/
/* program for lcau21 option 't'				*/
/* calculate probabilities related to evolution			*/
/* Harold V. McIntosh, 10 August 1987				*/
/* 26 March 1989 - options n, N for totalistic statistics	*/
/* 15 September 1989 - entropy calculations			*/
/* 5 May 1994 - adaptation to NeXTstep				*/

/* references:							*/
/*								*/
/* W. John Wilbur, David J. Lipman and Shihab A. Shamma		*/
/* On the prediction of local patterns in cellular automata	*/
/* Physica 19D 397-410 (1986)					*/ 
/*								*/
/* Howard A. Gutowitz, Jonathan D. Victor and Bruce W. Knight	*/
/* Local structure theory for cellular automata			*/
/* Physica 28D 18-48 (1987)					*/

/*	Copyright (C) 1987	*/
/*	Copyright (C) 1988	*/
/*	Copyright (C) 1989	*/
/*	Copyright (C) 1994	*/
/*	Harold V. McIntosh	*/

# include <math.h>
# include "nxlcau21.h"

extern char   ascrule[KK][KK][KK];
extern char   arr1[AL];
extern double blpr[KK];			/* probability from blocks	*/
extern double bp[BD];

# define EROW	    0	/* row for bar charts		*/
# define BROW	    0	/* row for bar charts		*/
# define AORG 	    0	/* x-origin  of contour plot    */
# define BORG 	    0	/* x-origin  of 2-block param   */
# define CORG 	    0	/* x-origin  of Bernstein plot  */

void   asctobin();
void   asczero();
void   edtri();
void   procur(int e, int g);
void   nblcur(int n);
void   trmenu();
void   pevolve();
void   nblclr();
void   gfram(int n, int l);
void   gdiag(int n, int l);
void   guare(int n, int l);
void   bgraf(int k, int n);
void   egraf(int n);
void   moncar(int n, int l);
void   neisum(int mm, int nn);
void   nbrsum(int mm, int nn);
void   gnlsum(int mm, int nn);
void   gnrsum(int mm, int b[KK][KK][KK]);
void   onesum(int mm, int cc, int *b);
void   berncoef();
void   bernthrd();
void   bernfrth();
void   bernfifth();
void   bernsixth();
double bern(double p, int n);
void   pdiff(int n);
void   pidiff(int n);
void   pjdiff(int n);
void   pdiff3(int n);
void   pdiff4(int n);
void   pdiff5(int n);
void   pdiff6(int n);
void   asfreq(int l);
void   lifreq(int n, int l);
void   twoblock(double *z);
void   twoblockfreq(int n);
void   onebl(double *x, double *a);
void   oneblfreq(int mm, int nn);
void   twobl(double x[][KK], double a[][KK]);
void   twoblfreq(int mm, int nn);
void   thrbl(double x[][KK][KK], double a[][KK][KK]);
void   thrblfreq(int mm, int nn);
void   foubl(double x[][KK][KK][KK], double a[][KK][KK][KK]);
void   foublfreq(int mm, int nn);
void   fivbl(double x[][KK][KK][KK][KK],
 	     double a[][KK][KK][KK][KK]);
void   fivblfreq(int mm, int nn);
void   sixbl(double x[][KK][KK][KK][KK][KK],
	     double a[][KK][KK][KK][KK][KK]);
void   sixblfreq(int mm, int nn);
void   svnbl(double x[][KK][KK][KK][KK][KK][KK], 
	     double a[][KK][KK][KK][KK][KK][KK]);
void   svnblfreq(int mm, int nn);
int    nearest(double ex,double wy);
void   onegen(int n);
void   videodot(int i, int j, int l);

/* edit the probability screen	*/

void edtri() {char c;
  videomode(COLGRAF);
  videopalette(YELREGR);

  while (0<1) {
  woruno(0,28);
  videocursor(0,0,36);
  videoputc('?',2);
  c=kbdin();
  if (c == '\015') break;
  videocursor(0,0,38);
  videoputc(c,2);
  videocursor(0,0,36);
  videoputc(' ',0);
  switch (c) {
    case '+': videopalette(WHCYMAG); break;
    case '-': videopalette(YELREGR); break;
    case 'a': asfreq(3); break;
    case 'e': pevolve(); break;
    case 'g': lifreq(50,2); break;
    case 'G': lifreq(200,1); break;
    case 'h': egraf(100); break;
    case 'm': moncar(1,2); break;
    case 'M': moncar(50,1); break;
    case 'n': nblclr(); neisum(AL,48); break;
    case 'N': nblclr(); nbrsum(AL,48); break;
    case 'x': pdiff(100); break;
    case 'i': pidiff(100); break;
    case 'j': pjdiff(100); break;
    case 'y': pdiff3(100); break;
    case 'z': pdiff4(100); break;
    case 'w': pdiff5(100); break;
    case 'v': pdiff6(100); break;
    case 't': twoblockfreq(100); break;
    case '1': nblclr(); oneblfreq(300,48); break;
    case '2': nblclr(); twoblfreq(300,48); break;
    case '3': nblclr(); thrblfreq(300,48); break;
    case '4': nblclr(); foublfreq(300,48); break;
    case '5': nblclr(); fivblfreq(300,48); break;
    case '6': nblclr(); sixblfreq(300,48); break;
    case '/': videomode(COLGRAF); videopalette(YELREGR); break;
    case '?': trmenu(); break;
    default: break;
    } /* end switch */
  }   /* end while  */
  videopalette(WHCYMAG);
  videomode(T80X25);
}      /* end edtri  */

/* show probability curves */
void procur(e,g) int e, g; {
  switch (e) {
    case 'e': switch (g) {
      case 0: gfram(100,3); gdiag(100,3); break;
      case 1: pdiff(100); break;
      case 2: pdiff3(100); break;
      case 3: pdiff4(100); break;
      case 4: pdiff5(100); break;
      case 5: pdiff6(100); break;
      default: break;} break;
    case 'i': switch (g) {
      case 0: gfram(100,3); gdiag(100,3); break;
      case 1: pdiff(100); break;
      case 2: pidiff(100); break;
      case 3: pjdiff(100); break;
      default: break;} break;
    default: break;}
}

/* show n-block probabilities */
void nblcur(n) int n; {switch (n) {
    case '1': oneblfreq(300,48); break;
    case '2': twoblfreq(300,48); break;
    case '3': thrblfreq(300,48); break;
    case '4': foublfreq(300,48); break;
    case '5': fivblfreq(300,48); break;
    case '6': sixblfreq(300,48); break;
    case '7': svnblfreq(300,48); break;
    default: break;}
}

/* show menu */
void trmenu() {
  videoscroll(BROW,0,BROW+8,40,0,0);
  videocursor(0,BROW,0);
  printf("a,h     - a priori estimates, entropy\n");
  printf("m,M,g,G - sample evolution\n");
  printf("xyzwv   - selfconsistent probabilities\n");
  printf("xij     - iterated s-c probabilities\n");
  printf("t       - graph 2 block probabilities\n");
  printf("123456  - n-block bar charts\n");
  printf("+-      - change color pallette\n");
  printf("e       - 12 lines evolution\n");
  printf("/?(clear screen, show menu), <cr>(exit)\n");
}

/* show fourteen lines of evolution at top of screen */
void pevolve() {int i, j;
  videoscroll(EROW,0,EROW+1,40,0,0);
  asctobin();
  for (j=8*EROW; j<8*(EROW+2)-2; j++) {
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
    }
}

/* Clear a space for the n-block frequencies */
void nblclr() {videoscroll(BROW,0,BROW+8,40,0,0);}

/* make a frame for a graph 		 */
/* n     = dimension of frame            */
/* l	 = color of frame		 */
void gfram(n,l) int n, l; {int i;

for (i=0; i<=n; i++) videodot(i,0,1);
for (i=0; i<=n; i++) videodot(i,n,1);
for (i=0; i<=n; i++) videodot(n,i,1);
for (i=0; i<=n; i++) videodot(0,i,1);

for (i=0; i<=n; i+=10) videodot(i,0,2);
for (i=0; i<=n; i+=10) videodot(i,n,2);
for (i=0; i<=n; i+=10) videodot(n,i,2);
for (i=0; i<=n; i+=10) videodot(0,i,2);
}

/* put a diagonal in a graph */

void gdiag(n, l) int n, l; {int i; for (i=0; 2*i<n; i++) videodot(2*i,2*i,l);}

/* place a graph of w=z*z at location x,y */

void guare(n,l) int n, l; {int i; for (i=0; i<=n; i+=2) videodot((i*i)/n,i,l);}

/* graph Bernstein polynomial */

void bgraf(k,n) int k, n; {int i; double bern(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  videodot((int)(en*bern(p,k)),i,1);
  }
}

/* graph entropy as a function of p	*/
/* n     number of points		*/

void egraf(n) int n; {int i; double el, em, en, p, q, s, t;
el=100.0/log(3.0);
em=100.0/log((double)KK);
en=1.0/(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))*en; q=1.0-p;
  s=p*log(p)+q*log(q/2.0);
  videodot(((int)(-s*el)),i,1);
  t=p*log(p)+q*log(q);
  videodot(((int)(-t*em)),i,0);
  }
}

double tdist(i,j,n) int i, j, n; { int a[10], b[10], l, m; double kay, s, t, u;
	if(n>10) return 0.0;
	kay=1.0/((double)KK);
	for(l=0; l<n; l++) a[l]=b[l]=0;
	for(m=i, l=0; l<n; m/=KK, l++) a[n-l-1]=m%KK;
	for(m=j, l=0; l<n; m/=KK, l++) b[n-l-1]=m%KK;
	for(s=0.0, t=1.0, l=0; l<n; t*=kay, l++){ u=a[l]-b[l]; s+=t*((u<0.0)?-u:u); }
	return s*kay;
	}

/* option 'm' */
/* "Monte Carlo" determination of probabilities */
/* n = mumber of generations			*/
/* l = color of dot				*/

void moncar(n,l) int n, l; {
int i, j, k, b[KK], bb[KK][KK];
double bf[KK], bbf[KK][KK];
double a[KK], aa[KK], d[KK], el, en, t, tt, s;

nblclr();
gfram(100,3);
guare(100,2);
asctobin();
el=1.0/((double)AL);
en=1.0/((double)n);
for (i=0; i<KK; i++) {a[i]=0.0; aa[i]=0.0;}
for (k=0; k<n; k++) {
  onegen(AL);
  for (i=0; i<KK; i++) b[i]=0;
  for (i=0; i<AL; i++) b[arr1[i]]++;
  for (i=0; i<KK; i++) bf[i]=((double)(b[i]))*el;
  for (i=0; i<KK; i++) {a[i]+=bf[i]; aa[i]+=bf[i]*bf[i];}
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) bb[i][j]=0;
  for (i=1; i<AL; i++) bb[arr1[i-1]][arr1[i]]++;
  bb[arr1[AL-1]][arr1[0]]++; 
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
    bbf[i][j]=((double)(bb[i][j]))*el;
  videodot((int)(100.0*bbf[1][1]),(int)(100.0*bf[1]),l);
  }
for (i=0; i<KK; i++) {a[i]*=en; aa[i]*=en;}
// videocursor(0,BROW+1,0);
s=1.0/log((double)KK);
t=0.0; tt=0.0;
for (i=0; i<KK; i++) t+=bf[i]*log(bf[i]);
for (i=0; i<KK; i++) d[i]=sqrt(aa[i]-a[i]*a[i]);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) tt+=bbf[i][j]*log(bbf[i][j]);
// printf("(Monte Carlo) \n"); 
// printf("lambda       = %5.3f\n",1.0-bf[0]);
// printf("entropy      = %5.3f\n",-t*s);
// printf("pair entropy = %5.3f\n",-0.5*tt*s);
// if (n>1) printf("average      = %5.3f, %5.3f\n",a[0],a[1]);
// if (n>1) printf("deviation    = %5.3f, %5.3f\n",d[0],d[1]);
// printf("frequencies ");
// for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,bf[i]);
// printf("\n");
// for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
//   printf("%1d%1d:%5.3f ",i,j,bbf[i][j]);
// printf("\n");
}

/* option 'n' (totalistic neighborhood) */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

void neisum(mm,nn) int mm, nn; {
int ii, i, k, m, n, l, b[TS];
double em, en, f, bf[TS], bft[TS];

asctobin();
em=((double)(mm));
// f=1.0;
f=115.0/((double)(AL));
for (i=0; i<TS; i++) bft[i]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  onesum(mm,TS,b);
  for (i=0; i<TS; i++) bf[i]=((double)(b[i]));
  for (i=0; i<TS; i++) bft[i]+=bf[i];
  m=0; videodot(ii,m++,1);
  for (i=0; i<TS; i++) {
    n=(int)(f*bf[i]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i%2);
    }
  videodot(ii,m++,1);
  k++;
  }  /* end for-ii */
  en=((double)(k));
  m=0; videodot(nn,m++,1);
  for (i=0; i<TS; i++) {
    n=(int)(0.1*(bft[i]/en));
    if (n>0) for (l=0; l<n; l++) videodot(nn,m++,i%2);
    }
  videodot(nn,m++,1); /* cambio 3 por 1*/
// for (i=0; i<TS; i++) bft[i]/=en*em;
// videocursor(0,BROW+7,0);
// for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* option 'n' (general neighborhood) */
/* empirical distribution of sums within a neighborhood */
/* mm - number of cells	*/
/* nn - number of lines	*/

void gnlsum(mm,nn) int mm, nn; {
int    i0, i1, i2;
int    ii, k, m, n, l, b[KK][KK][KK];
double em, en, f, bf[KK][KK][KK], bft[KK][KK][KK];

asctobin();
em=((double)(mm));
// f=1.0;
f=115.0/((double)(AL));
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bft[i0][i1][i2]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  gnrsum(mm,b);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bf[i0][i1][i2]=((double)(b[i0][i1][i2]));
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bft[i0][i1][i2]+=bf[i0][i1][i2];
  m=0; videodot(ii,m++,1);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) {
    n=(int)(f*bf[i0][i1][i2]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i2%2);
    }
  videodot(ii,m++,1);
  k++;
  }  /* end for-ii */
  en=((double)(k));
  m=0; videodot(nn,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) {
    n=(int)(0.1*(bft[i0][i1][i2]/en));
    if (n>0) for (l=0; l<n; l++) videodot(nn,m++,i0%2+1);
    }
  videodot(nn,m++,3);
}

/* option 'N' */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/
void nbrsum(mm,nn) int mm, nn; {
int ii, jj, i, k, m, n, l, b[TS];
double em, en, kk, bf[TS], bft[TS];

asctobin();
em=((double)(mm));
en=((double)(nn));
for (i=0; i<TS; i++) bft[i]=0.0;
k=0;
for (jj=0; jj<nn; jj++) {
  for (i=0; i<TS; i++) bf[i]=0.0;
  for (ii=0; ii<nn; ii++) {
    onegen(mm);
    onesum(mm,TS,b);
    for (i=0; i<TS; i++) bf[i]+=((double)(b[i]));
    }  /* end for-ii */
  m=0; videodot(jj,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bf[i]/en);
    if (n>0) for (l=0; l<n; l++) videodot(jj,m++,i%2+1);
    }
  videodot(jj,m++,3);
  for (i=0; i<TS; i++) bft[i]+=bf[i];
  k++;
  } /* end for-jj */
kk=((double)(k));
videodot(jj,m++,3);
for (i=0; i<TS; i++) bft[i]/=kk*en*em;
// videocursor(0,BROW+7,0);
for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* gather totalistic neighborhood sums for one generation   */
/* in other words, a histogram for the number of times each */
/* sum has occurred in neighborhoods on the given line      */
/* mm = number of cells 				    */
/* b[cc]  = sum counts  				    */
void onesum(mm,cc,b) int mm, cc, *b; {int i, j, k;
  for (i=0; i<cc; i++) b[i]=0;
  k=arr1[mm-1]+arr1[0]+arr1[1]; b[k]++;
  for (i=1; i<mm-2; i++) {k=0; for (j=i-1; j<i+2; j++) k+=arr1[j]; b[k]++;}
  k=arr1[mm-2]+arr1[mm-1]+arr1[0]; b[k]++;
}

/* gather general neighborhood sums for one generation */
/* in other words, a histogram for the number of times */
/* each neighborhood has occurred in the given line    */
/* mm = number of cells         		       */
/* b[KK][KK][KK]  = sum counts  		       */
void gnrsum(mm,b) int mm, b[KK][KK][KK]; {int i0, i1, i2, i;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	b[i0][i1][i2]=0;
  b[arr1[mm-1]][arr1[0]][arr1[1]]++;
  for (i=1; i<mm-2; i++) b[arr1[i-1]][arr1[i]][arr1[i+1]]++;
  b[arr1[mm-2]][arr1[mm-1]][arr1[0]]++;
}

/* Generate coefficients of 2nd generation Bernstein Polynomial */
void berncoef() {
int i, i0, i1, i2;

for (i=0; i<BD; i++) bp[i]=0.0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++) {
  if (ascrule[i0][i1][i2]=='1') bp[i0+i1+i2]+=1.0;
  }
}

/* Generate coefficients of 3rd generation Bernstein Polynomial */
void bernthrd() {
int i, i0, i1, i2, i3, i4;
int j0, j1, j2;

for (i=0; i<BD; i++) bp[i]=0.0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  if (ascrule[j0][j1][j2]=='1') bp[i0+i1+i2+i3+i4]+=1.0;
  }
}

/* Generate coefficients of 4th generation Bernstein Polynomial */
void bernfrth() {
int i, i0, i1, i2, i3, i4, i5, i6;
int j0, j1, j2, j3, j4;
int k0, k1, k2;

for (i=0; i<BD; i++) bp[i]=0.0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  j3=ascrule[i3][i4][i5]-'0';
  j4=ascrule[i4][i5][i6]-'0';
  k0=ascrule[j0][j1][j2]-'0';
  k1=ascrule[j1][j2][j3]-'0';
  k2=ascrule[j2][j3][j4]-'0';
  if (ascrule[k0][k1][k2]=='1') bp[i0+i1+i2+i3+i4+i5+i6]+=1.0;
  }
}

/* Generate coefficients of 5th generation Bernstein Polynomial */
void bernfifth() {
int i, i0, i1, i2, i3, i4, i5, i6, i7, i8;
int j0, j1, j2, j3, j4, j5, j6;
int k0, k1, k2, k3, k4;
int l0, l1, l2;

for (i=0; i<BD; i++) bp[i]=0.0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  j3=ascrule[i3][i4][i5]-'0';
  j4=ascrule[i4][i5][i6]-'0';
  j5=ascrule[i5][i6][i7]-'0';
  j6=ascrule[i6][i7][i8]-'0';
  k0=ascrule[j0][j1][j2]-'0';
  k1=ascrule[j1][j2][j3]-'0';
  k2=ascrule[j2][j3][j4]-'0';
  k3=ascrule[j3][j4][j5]-'0';
  k4=ascrule[j4][j5][j6]-'0';
  l0=ascrule[k0][k1][k2]-'0';
  l1=ascrule[k1][k2][k3]-'0';
  l2=ascrule[k2][k3][k4]-'0';
  if (ascrule[l0][l1][l2]=='1') bp[i0+i1+i2+i3+i4+i5+i6+i7+i8]+=1.0;
  }
}

/* Generate coefficients of 6th generation Bernstein Polynomial */
void bernsixth() {
int i, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10;
int j0, j1, j2, j3, j4, j5, j6, j7, j8;
int k0, k1, k2, k3, k4, k5, k6;
int l0, l1, l2, l3, l4;
int m0, m1, m2;

for (i=0; i<BD; i++) bp[i]=0.0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (i10=0; i10<KK; i10++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  j3=ascrule[i3][i4][i5]-'0';
  j4=ascrule[i4][i5][i6]-'0';
  j5=ascrule[i5][i6][i7]-'0';
  j6=ascrule[i6][i7][i8]-'0';
  j7=ascrule[i7][i8][i9]-'0';
  j8=ascrule[i8][i9][i10]-'0';
  k0=ascrule[j0][j1][j2]-'0';
  k1=ascrule[j1][j2][j3]-'0';
  k2=ascrule[j2][j3][j4]-'0';
  k3=ascrule[j3][j4][j5]-'0';
  k4=ascrule[j4][j5][j6]-'0';
  k5=ascrule[j5][j6][j7]-'0';
  k6=ascrule[j6][j7][j8]-'0';
  l0=ascrule[k0][k1][k2]-'0';
  l1=ascrule[k1][k2][k3]-'0';
  l2=ascrule[k2][k3][k4]-'0';
  l3=ascrule[k3][k4][k5]-'0';
  l4=ascrule[k4][k5][k6]-'0';
  m0=ascrule[l0][l1][l2]-'0';
  m1=ascrule[l1][l2][l3]-'0';
  m2=ascrule[l2][l3][l4]-'0';
  if (ascrule[m0][m1][m2]=='1') bp[i0+i1+i2+i3+i4+i5+i6+i7+i8+i9+i10]+=1.0;
  }
}

/* evaluate the nth generation Bernstein polynomial at point p */
double bern(p,n) double p; int n; {double q, s, x, r; int i, d;
d=2*n-1;
if (p>0.99) return bp[d];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<d;  i++) x*=q;
for (i=0; i<=d; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* graph the probability of the second generation */
void pdiff(n) int n; {
berncoef();
gfram(n,3);
gdiag(n,3);
bgraf(2,n);
}

/* graph the iterated probability of the second generation */
void pidiff(n) int n; {int i; double bern(), en, p;
en=1.0/((double)(n));
berncoef();
gfram(n,3);
gdiag(n,3);
for (i=0; i<=n; i++) {
  p=((double)(i))*en;
  videodot((int)(100.0*bern(bern(p,2),2)),(int)(100.0*p),3);
  }
}

/* graph the second iterated probability of the second generation */
void pjdiff(n) int n; {int i; double bern(), en, p;
en=1.0/((double)(n));
berncoef();
gfram(n,3);
gdiag(n,3);
for (i=0; i<=n; i++) {
  p=((double)(i))*en;
  videodot((int)(100.0*bern(bern(bern(p,2),2),2)),(int)(100.0*p),3);
  }
}

/* graph the probability of the third generation */
void pdiff3(n) int n; {
bernthrd();
gfram(n,3);
gdiag(n,3);
bgraf(3,n);
}

/* graph the probability of the fourth generation */
void pdiff4(n) int n; {
bernfrth();
gfram(n,3);
gdiag(n,3);
bgraf(4,n);
}

/* graph the probability of the fifth generation */
void pdiff5(n) int n; {
bernfifth();
gfram(n,3);
gdiag(n,3);
bgraf(5,n);
}

/* graph the probability of the sixth generation */

void pdiff6(n) int n; {
bernsixth();
gfram(n,3);
gdiag(n,3);
bgraf(6,n);
}

/* graph the frequencies of ascrule in color l	 */
/* put dot at coordinates (u,v)			 */
/* percentages evolving into each value		 */
/* percentages shifting - adds to more than 100% */
/* entropy and Langton's lambda = % nonquiescent */

void asfreq(l) int l; {
int    i, j, i0, i1, i2;
int    stat[KK], stal, stac, star;		/* statistic counts */
double staf[KK], t, pp;

videoscroll(BROW,0,BROW+8,40,0,0);
videocursor(0,BROW,0);
pp=1.0/((double)(KK*KK*KK));
stal=0; stac=0; star=0;
for (i=0; i<KK; i++) stat[i]=0;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
  j=ascrule[i0][i1][i2]-'0';
  stat[j]++;
  if(j==i0) star++;
  if(j==i1) stac++;
  if(j==i2) stal++;
  }}}
videocursor(0,BROW,0);
for (i=0; i<KK; i++) staf[i]=((double)stat[i])*pp;
for (i=0; i<KK; i++) printf("%d freq - %5.2f\n",i,staf[i]);
printf("\n");
printf("left   - %5.2f\n",((double)stal)*pp);
printf("still  - %5.2f\n",((double)stac)*pp);
printf("right  - %5.2f\n",((double)star)*pp);
j=(int)(100.0*staf[1]);
i=(int)(100.0*staf[1]);
if (j>=2) videodot(j-2,i,l);
if (i>=2) videodot(j,i-2,l);
videodot(j,i,1);
if (i<=318) videodot(j,i+2,l);
if (j<=198) videodot(j+2,i,l);
for (i=0, t=0.0; i<KK; i++) t+=staf[i]*log(staf[i]);
videocursor(0,BROW,20);
printf("lambda:   %5.3f",1.0-staf[0]);
videocursor(0,BROW+1,20);
printf("entropy:  %5.3f",-t/log((double)KK));
}

/* display state frequencies in arr1 as a point on probability graph */
/* n = number of points to plot */
/* l = color of plotted point   */

void lifreq(n,l) int n, l; {
int    i, j, os, ns;
double of, nf, s;

s=100.0/((double)AL);
asctobin();
for (i=0; i<n; i++) {
  os=0; ns=0;
  for (j=0; j<AL; j++) if (arr1[j]==1) os++;
  onegen(AL);
  for (j=0; j<AL; j++) if (arr1[j]==1) ns++;
  of=(double)os;
  nf=(double)ns;
  videodot((int)(s*nf),(int)(s*of),l);
  }
}

/* evaluate the parameters for two-block probabilities after one generation */

void twoblock(z) double *z; {
int    i, i0, i1, i2, i3;
int    j, j0, j1;
double w, x, y, p, q, pp;
double a[KK][KK], b[KK], c[KK][KK];

p=z[0]; q=1.0-p; pp=z[1];
a[1][1]=pp; a[1][0]=a[0][1]=p-pp; a[0][0]=1.0-2*p+pp;
b[1]=p; b[0]=q;
for (i=0; i<KK; i++) {for (j=0; j<KK; j++) c[i][j]=0.0;}
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  x=a[i0][i1]*a[i1][i2]*a[i2][i3];
  y=b[i1]*b[i2];
  w=0.0; if (y>0.00001) w=x/y;
  c[j0][j1]+=w;
  }
z[0]=c[1][0]+c[1][1]; z[1]=c[1][1];
}

/* graph the 2-block parameter differences to find self-consistent values */
/* nxn points 								  */

void twoblockfreq(n) int n; {
int    i, j, l;
double op[2], np[2], s, t, u, v;

gfram(100,3);
s=1.0/((double)(n));
for (op[1]=s, i=1; i<n; op[1]+=s, i++) {
  for (op[0]=s, j=1; j<n; op[0]+=s, j++) {
    np[0]=op[0]; np[1]=op[1]; twoblock(np);
    u=np[0]-op[0]; v=np[1]-op[1];
    t=u*u+v*v;
    if (t<0.0025) l=0; else {
    if (t<0.0675) {if((i+j)%2) l=0; else l=3;} else {
    if (t<0.3906) l=3; else {
    if (t<0.7500) {if((i+j)%2) l=3; else l=2;} else {
    if (t<1.5000) l=2; else {
    if (t<3.0000) {if((i+j)%2) l=2; else l=1;} else {
    if (t<6.0000) l=1; else {
                  if((i+j)%2) l=1; else l=0;
       }}}}}}}
    videodot(i,j,l);
    }  /* end for-j */
  }    /* end for-i */
}

/* evaluate the one-block probabilities after one generation */
void onebl(x,a) double x[KK], a[KK]; {int i0, i1, i2, j0;

for (j0=0; j0<KK; j0++) x[j0]=0.0;

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++) {
  j0=ascrule[i0][i1][i2]-'0';
  x[j0]+=a[i0]*a[i1]*a[i2];
  }
}

/* iterate the 1-block parameters to find self-consistent values */
/* graph the iterative steps in bar-chart form			 */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/
void oneblfreq(mm,nn) int mm, nn; {
int    ii, i, l, m, n;
double op[KK], np[KK];
double d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i=0; i<KK; i++) {op[i]=s; for (l=0; l<n; l++) videodot(0,m++,i);}

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  onebl(np,op);
  videodot(ii,m++,3);
  for (i=0; i<KK; i++) {
    n=(int)(f*np[i]); if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i);
    d=op[i]-np[i];
    e+=d*d;
    op[i]=np[i];
    }
  videodot(ii,m++,3);
  if (op[0]<=0.001) break;
  if (op[1]<=0.001) break;
  if (e<=0.0000001) break;
  }
// videocursor(0,BROW+7,0);
// printf("(1-block) "); 
// for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,op[i]);
}

/* evaluate the two-block probabilities after one generation */
void twobl(x,a) double x[KK][KK], a[KK][KK]; {
int    i0, i1, i2, i3;
int    j0, j1;
double w, b[KK];

for (j0=0; j0<KK; j0++) {for (j1=0; j1<KK; j1++) x[j0][j1]=0.0;}

for (j0=0; j0<KK; j0++) {b[j0]=0.0; for (j1=0; j1<KK; j1++) b[j0]+=a[j0][j1];}

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  w=a[i0][i1]*a[i1][i2]*a[i2][i3];
  if (w!=0.0) w/=b[i1]*b[i2];
  x[j0][j1]+=w;
  }
}

/* iterate the 2-block parameters to find self-consistent values */
/* graph the iterative steps in bar-chart form			 */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/
void twoblfreq(mm,nn) int mm, nn; {
int    ii, i, j, l, m, n;
double op[KK][KK], np[KK][KK];
double d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i=0; i<KK; i++)
for (j=0; j<KK; j++) {
  op[i][j]=s;
  for (l=0; l<n; l++) videodot(0,m++,j);
  }
videodot(0,m++,3);

for (i=0; i<KK; i++) {blpr[i]=0.0; for (j=0; j<KK; j++) blpr[i]+=op[i][j];}
videodot((int)(100.0*op[1][1]),(int)(100.0*blpr[1]),1);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  twobl(np,op);
  videodot(ii,m++,3);
  for (i=0; i<KK; i++)
  for (j=0; j<KK; j++) {
    n=(int)(f*np[i][j]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,j);
    d=op[i][j]-np[i][j];
    e+=d<0.0?-d:d;
    op[i][j]=np[i][j];
    }
  videodot(ii,m++,3);
  for (i=0; i<KK; i++) {blpr[i]=0.0; for (j=0; j<KK; j++) blpr[i]+=op[i][j];}
  videodot((int)(100.0*op[1][1]),(int)(100.0*blpr[1]),2);
  if (e<=0.00001) break;
  }
// videocursor(0,BROW+7,0);
// printf("(2-block) "); 
// for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,blpr[i]);
// videocursor(0,11,0);
// for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
// 		printf("%1d%1d:%5.3f ",i,j,op[i][j]);
videodot((int)(100.0*op[1][1]),(int)(100.0*blpr[1]),3);
videodot((int)(100.0*op[1][1]),(int)(100.0*blpr[1]),1);
}

/* evaluate the three-block probabilities after one generation */
void thrbl(x,a) double x[KK][KK][KK], a[KK][KK][KK]; {
int    i0, i1, i2, i3, i4;
int    j0, j1, j2;
double w, b[KK][KK];

for (j0=0; j0<KK; j0++)
for (j1=0; j1<KK; j1++) {
  for (j2=0; j2<KK; j2++) x[j0][j1][j2]=0.0;
  b[j0][j1]=0.0;
  for (j2=0; j2<KK; j2++) b[j0][j1]+=a[j0][j1][j2];  
  }

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  w=a[i0][i1][i2]*a[i1][i2][i3]*a[i2][i3][i4];
  if (w!=0.0) w/=b[i1][i2]*b[i2][i3];
  x[j0][j1][j2]+=w;
  }
}

/* iterate the 3-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/
void thrblfreq(mm,nn) int mm, nn; {
int    ii, i, j, k, l, m, n;
double op[KK][KK][KK], np[KK][KK][KK];
double bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i=0; i<KK; i++)
for (j=0; j<KK; j++)
for (k=0; k<KK; k++) {
  op[i][j][k]=s;
  for (l=0; l<n; l++) videodot(0,m++,k);
  }
videodot(0,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  thrbl(np,op);
  videodot(ii,m++,3);
  for (i=0; i<KK; i++)
  for (j=0; j<KK; j++)
  for (k=0; k<KK; k++) {
    n=(int)(f*np[i][j][k]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,k);
    d=op[i][j][k]-np[i][j][k];
    e+=d<0.0?-d:d;
    op[i][j][k]=np[i][j][k];
    }
  videodot(ii,m++,3);
  if (e<=0.00001) break;
  }

for (i=0; i<KK; i++) {
  blpr[i]=0.0;
  for (j=0; j<KK; j++)
  for (k=0; k<KK; k++) {
    blpr[i]+=op[i][j][k];
    }
  }
// videocursor(0,BROW+7,0);
// printf("(3-block) "); 
// for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,blpr[i]);
// printf("\n");

for (i=0; i<KK; i++)
for (j=0; j<KK; j++) {
  bb[i][j]=0.0;
  for (k=0; k<KK; k++) {
    bb[i][j]+=op[i][j][k];
    }
  }

// videocursor(0,11,0);
// for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
//   printf("%1d%1d:%5.3f ",i,j,bb[i][j]);
// videodot((int)(100.0*bb[1][1]),(int)(100.0*blpr[1]),2);
}

/* evaluate the four-block probabilities after one generation */
void foubl(x,a) double x[KK][KK][KK][KK], a[KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5;
int    j0, j1, j2, j3;
double w, b[KK][KK][KK];

for (j0=0; j0<KK; j0++)
for (j1=0; j1<KK; j1++)
for (j2=0; j2<KK; j2++) {
  for (j3=0; j3<KK; j3++) x[j0][j1][j2][j3]=0.0;
  b[j0][j1][j2]=0.0;
  for (j3=0; j3<KK; j3++) b[j0][j1][j2]+=a[j0][j1][j2][j3];  
  }

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  j3=ascrule[i3][i4][i5]-'0';
  w=a[i0][i1][i2][i3]*a[i1][i2][i3][i4]*a[i2][i3][i4][i5];
  if (w!=0.0) w/=b[i1][i2][i3]*b[i2][i3][i4];
  x[j0][j1][j2][j3]+=w;
  }
}

/* iterate the 4-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/
void foublfreq(mm,nn) int mm, nn; {
int    ii, i0, i1, i2, i3, l, m, n;
double op[KK][KK][KK][KK], np[KK][KK][KK][KK];
double bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  op[i0][i1][i2][i3]=s;
  for (l=0; l<n; l++) videodot(0,m++,i3);
  }
videodot(0,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  foubl(np,op);
  videodot(ii,m++,3);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++) {
    n=(int)(f*np[i0][i1][i2][i3]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i3);
    d=op[i0][i1][i2][i3]-np[i0][i1][i2][i3];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3]=np[i0][i1][i2][i3];
    }
  videodot(ii,m++,3);
  if (e<=0.00001) break;
  }

for (i0=0; i0<KK; i0++) {
  blpr[i0]=0.0;
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++) {
    blpr[i0]+=op[i0][i1][i2][i3];
    } }

// videocursor(0,BROW+7,0);
// printf("(4-block) "); 
// for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,blpr[i0]);

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
    bb[i0][i1]+=op[i0][i1][i2][i3];
    }} }}

// videocursor(0,11,0);
// for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
//   printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
videodot((int)(100.0*bb[1][1]),(int)(100.0*blpr[1]),2);
}

/* evaluate the five-block probabilities after one generation */
void fivbl(x,a) double x[KK][KK][KK][KK][KK], a[KK][KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6;
int    j0, j1, j2, j3, j4;
double w, b[KK][KK][KK][KK];

for (j0=0; j0<KK; j0++)
for (j1=0; j1<KK; j1++)
for (j2=0; j2<KK; j2++)
for (j3=0; j3<KK; j3++) {
  for (j4=0; j4<KK; j4++) x[j0][j1][j2][j3][j4]=0.0;
  b[j0][j1][j2][j3]=0.0;
  for (j4=0; j4<KK; j4++) b[j0][j1][j2][j3]+=a[j0][j1][j2][j3][j4];  
  }

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  j3=ascrule[i3][i4][i5]-'0';
  j4=ascrule[i4][i5][i6]-'0';
  w=a[i0][i1][i2][i3][i4]*a[i1][i2][i3][i4][i5]*a[i2][i3][i4][i5][i6];
  if (w!=0.0) w/=b[i1][i2][i3][i4]*b[i2][i3][i4][i5];
  x[j0][j1][j2][j3][j4]+=w;
  }
}

/* iterate the 5-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/
void fivblfreq(mm,nn) int mm, nn; {
int    ii, i0, i1, i2, i3, i4, l, m, n;
double op[KK][KK][KK][KK][KK], np[KK][KK][KK][KK][KK];
double bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  op[i0][i1][i2][i3][i4]=s;
  for (l=0; l<n; l++) videodot(0,m++,i4);
  }
videodot(0,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  fivbl(np,op);
  videodot(ii,m++,3);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
    n=(int)(f*np[i0][i1][i2][i3][i4]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i4);
    d=op[i0][i1][i2][i3][i4]-np[i0][i1][i2][i3][i4];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3][i4]=np[i0][i1][i2][i3][i4];
    }
  videodot(ii,m++,3);
  if (e<=0.00001) break;
  }
for (i0=0; i0<KK; i0++) {
  blpr[i0]=0.0;
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
    blpr[i0]+=op[i0][i1][i2][i3][i4];
    }
  }
// videocursor(0,BROW+7,0);
// printf("(5-block) "); 
// for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,blpr[i0]);

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
    bb[i0][i1]+=op[i0][i1][i2][i3][i4];
    }
  }

// videocursor(0,11,0);
// for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
//   printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
// videodot((int)(100.0*bb[1][1]),(int)(100.0*blpr[1]),2);
}

/* evaluate the six-block probabilities after one generation */
void sixbl(x,a) double x[KK][KK][KK][KK][KK][KK], a[KK][KK][KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7;
int    j0, j1, j2, j3, j4, j5;
double w, b[KK][KK][KK][KK][KK];

for (j0=0; j0<KK; j0++)
for (j1=0; j1<KK; j1++)
for (j2=0; j2<KK; j2++)
for (j3=0; j3<KK; j3++)
for (j4=0; j4<KK; j4++) {
  for (j5=0; j5<KK; j5++) x[j0][j1][j2][j3][j4][j5]=0.0;
  b[j0][j1][j2][j3][j4]=0.0;
  for (j5=0; j5<KK; j5++) b[j0][j1][j2][j3][j4]+=a[j0][j1][j2][j3][j4][j5];  
  }

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  j3=ascrule[i3][i4][i5]-'0';
  j4=ascrule[i4][i5][i6]-'0';
  j5=ascrule[i5][i6][i7]-'0';
  w=a[i0][i1][i2][i3][i4][i5]*a[i1][i2][i3][i4][i5][i6]*a[i2][i3][i4][i5][i6][i7];
  if (w!=0.0) w/=b[i1][i2][i3][i4][i5]*b[i2][i3][i4][i5][i6];
  x[j0][j1][j2][j3][j4][j5]+=w;
  }
}

/* iterate the 6-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/
void sixblfreq(mm,nn) int mm, nn; {
int    ii, i0, i1, i2, i3, i4, i5, l, m, n;
double op[KK][KK][KK][KK][KK][KK];
double np[KK][KK][KK][KK][KK][KK];
double bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK*KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++) {
  op[i0][i1][i2][i3][i4][i5]=s;
  for (l=0; l<n; l++) videodot(0,m++,i5);
  }
videodot(0,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  sixbl(np,op);
  videodot(ii,m++,3);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++) {
    n=(int)(f*np[i0][i1][i2][i3][i4][i5]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i5);
    d=op[i0][i1][i2][i3][i4][i5]-np[i0][i1][i2][i3][i4][i5];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3][i4][i5]=np[i0][i1][i2][i3][i4][i5];
    }
  videodot(ii,m++,3);
  if (e<=0.00001) break;
  }
for (i0=0; i0<KK; i0++) {
  blpr[i0]=0.0;
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++) {
    blpr[i0]+=op[i0][i1][i2][i3][i4][i5];
    }
  }
// videocursor(0,BROW+7,0);
// printf("(6-block) "); 
// for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,blpr[i0]);

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++) {
    bb[i0][i1]+=op[i0][i1][i2][i3][i4][i5];
    }
  }

// videocursor(0,11,0);
// for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
//   printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
// videodot((int)(100.0*bb[1][1]),(int)(100.0*blpr[1]),2);
}

/* evaluate the seven-block probabilities after one generation */
void svnbl(x,a) double x[KK][KK][KK][KK][KK][KK][KK], 
			a[KK][KK][KK][KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7, i8;
int    j0, j1, j2, j3, j4, j5, j6;
double w, b[KK][KK][KK][KK][KK][KK] ;

for (j0=0; j0<KK; j0++)
for (j1=0; j1<KK; j1++)
for (j2=0; j2<KK; j2++)
for (j3=0; j3<KK; j3++)
for (j4=0; j4<KK; j4++)
for (j5=0; j5<KK; j5++) {
  for (j6=0; j6<KK; j6++) x[j0][j1][j2][j3][j4][j5][j6]=0.0;
  b[j0][j1][j2][j3][j4][j5]=0.0;
  for (j6=0; j6<KK; j6++) b[j0][j1][j2][j3][j4][j5]+=a[j0][j1][j2][j3][j4][j5][j6];  
  }

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
  j0=ascrule[i0][i1][i2]-'0';
  j1=ascrule[i1][i2][i3]-'0';
  j2=ascrule[i2][i3][i4]-'0';
  j3=ascrule[i3][i4][i5]-'0';
  j4=ascrule[i4][i5][i6]-'0';
  j5=ascrule[i5][i6][i7]-'0';
  j6=ascrule[i6][i7][i8]-'0';

w=a[i0][i1][i2][i3][i4][i5][i6]*a[i1][i2][i3][i4][i5][i6][i7]*a[i2][i3][i4][i5][i6][i7][i8];

  if (w!=0.0) w/=b[i1][i2][i3][i4][i5][i6]*b[i2][i3][i4][i5][i6][i7];
  x[j0][j1][j2][j3][j4][j5][j6]+=w;
  }
}

/* iterate the 7-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/
void svnblfreq(mm,nn) int mm, nn; {
int    ii, i0, i1, i2, i3, i4, i5, i6, l, m, n;
double op[KK][KK][KK][KK][KK][KK][KK];
double np[KK][KK][KK][KK][KK][KK][KK];
double bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK*KK*KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  op[i0][i1][i2][i3][i4][i5][i6]=s;
  for (l=0; l<n; l++) videodot(0,m++,i6);
  }
videodot(0,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  svnbl(np,op);
  videodot(ii,m++,3);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
    n=(int)(f*np[i0][i1][i2][i3][i4][i5][i6]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i6);
    d=op[i0][i1][i2][i3][i4][i5][i6]-np[i0][i1][i2][i3][i4][i5][i6];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3][i4][i5][i6]=np[i0][i1][i2][i3][i4][i5][i6];
    }
  videodot(ii,m++,3);
  if (e<=0.00001) break;
  }
for (i0=0; i0<KK; i0++) {
  blpr[i0]=0.0;
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
    blpr[i0]+=op[i0][i1][i2][i3][i4][i5][i6];
    }
  }
// videocursor(0,BROW+7,0);
// printf("(7-block) "); 
// for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,blpr[i0]);

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
    bb[i0][i1]+=op[i0][i1][i2][i3][i4][i5][i6];
    }
  }

// videocursor(0,11,0);
// for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
//   printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
// videodot((int)(100.0*bb[1][1]),(int)(100.0*blpr[1]),2);
}

/* end nxprob21.c */
