
/* Mandatory declarations */

# include "rec.h"


int r_lpar();		/* compilation of a left parenthesis */
int r_rpar();		/* compilation of a right parenthesis */
int r_colon();		/* compilation of a colon */
int r_semicol();	/* compilation of a semicolon */
int r_code();		/* compilation of a simple operator */
int r_oper1();		/* compilation of an operator with an ASCII arg */
int r_pred();		/* compilation of a simple predicate*/
int r_pred1();		/* compilation of a predicate with an ASCII arg */
int r_noopc();		/* compilation of no-op */
int r_comment();	/* compilation of a (bracket enclosed) comment */
int r_ubrack();	/* compilation of an unbalanced right bracket */
int r_lbrace();	/* compilation of a (brace enclosed) program */
int r_ubrace();	/* compilation of an unbalanced right brace */

int r_call();		/* execution of predicate @x (call subr. x) */
int r_quit();		/* execution of operator _ (exit to O.S.) */
int r_xbrace();		/* execution of brace-enclosed program */

/* Optional: Counter predicate */

int r_ctrc();		/* compilation of a counter */
int r_ctrx();		/* execution of a counter */

/* opcional declarations, needed only in versions using quotes and
   numbers ( see cnum.c and cquo.c ); r_ld, r_ldch, and r_xstr must 
   be provided by user */

int r_dquote(); /* compile doubly quoted string into simbol table */ 
int r_squote(); /* compile singly quoted char into the program array*/ 
int r_cmin(); /* compile '-' as simple operator, call r_cnum if followed
                by digit or period */ 
int r_cnum(); /* compile a number (WORD, LONG or REAL) into sym. table */ 

int r_ld(); /* execution of compiled numbers */ 
int r_ldch(); /* execution of apostrophe (single quote op.) */ 
int r_xstr(); /* execution of quoted strings */
/**************************/
/* Optional: Integer parameter operator  (module INT.C) */

int r_cintp();		/* compilation of an integer parameter */
int r_lintp();		/* execution (loading) of an integer parameter */

/* Optional: Double parameter operator (module DBL.C) */

int r_cdblp();		/* compilation of a double parameter */
int r_ldblp();		/* execution (loading) of a double parameter */

/* Optional: String parameter operator (module STR.C) */

int r_cstrp();		/* compilation of a string parameter */
int r_lstrp();		/* execution (loading) of a string parameter */

/* Optional: Character parameter operator (module CHR.C) */

int r_cchrp();		/* compilation of a character parameter */
int r_lchrp();		/* execution (loading) of a character parameter */

/* Declarations of user-provided execution subroutines */

void rop0(), rop1(), roplz();
Inst roppc(), roplr(), ropll();
void ropuw();
/**************************/
 
/* Declarations of user-provided execution subroutines */ 

/* Table of compiling/executing function definitions,
   a pair for each printing ASCII character */

struct fptbl dtbl[] = {
        r_noopc,        False,          "[blank]",  
	r_ctrc,		r_ctrx,	        " !  - Counter !n!, n decimal ",
	r_noopc,	False,		" comilla  - quoted string ",	
	r_noopc,	False,		" #  - ",
	r_noopc,	False,		" $  - ",
        r_pred,         roppc,          "%   - 50% T predicate ",
	r_noopc,	False,		" &  - random perturbations ",
	r_noopc,	False,		" '  - quoted char ",
	r_lpar,		False,		" (  - start of expression ",
	r_rpar,		False,		" )  - end of expression ",
	r_noopc,	False,		" *  - ",
	r_noopc,	False,		" +  - ",
	r_noopc,	False,		" ,  - separator",
	r_noopc,	False,		" -  - ",
	r_noopc,	False,		" .  - ",
	r_noopc,	False,		" /  - ",
        r_code,         rop0,           "0   - set state 0 ",
        r_code,         rop1,           "1   - set state 1 ",
	r_noopc,	False,		" 2  - ",
	r_noopc,	False,		" 3  - ",
	r_noopc,	False,		" 4  - ",
	r_noopc,	False,		" 5  - ",
	r_noopc,	False,		" 6  - ",
	r_noopc,	False,		" 7  - ",
	r_noopc,	False,		" 8  - ",
	r_noopc,	False,		" 9  - ",
	r_colon,	False,		" :  - ",
	r_semicol,	False,		" ;  - ",
	r_noopc,	False,		" <  - ",
	r_noopc,	False,		" =  - ",
	r_noopc,	False,		" >  - ",
	r_noopc,	False,		" ?  - ",
	r_pred1,	r_call,		" @  - call a subroutine ",
	r_noopc,	False,		" A  - ",
	r_noopc,	False,		" B  - ",
	r_noopc,	False,		" C  - ",
	r_noopc,	False,		" D  - ",
	r_noopc,	False,		" E  - ",
	r_noopc,	False,		" F  - ",
	r_noopc,	False,		" G  - ",
	r_noopc,	False,		" H  - ",
	r_noopc,	False,		" I  - ",
	r_noopc,	False,		" J  - ",
	r_noopc,	False,		" K  - ",
	r_noopc,	False,		" L  - ",
	r_noopc,	False,		" M  - ",
	r_noopc,	False,		" N  - ",
	r_noopc,	False,		" O  - ",
	r_noopc,	False,		" P  - ",
	r_noopc,	False,		" Q  - ",
	r_noopc,	False,		" R  - ",
	r_noopc,	False,		" S  - ",
	r_noopc,	False,		" T  - ",
	r_noopc,	False,		" U  - ",
	r_noopc,	False,		" V  - ",
        r_code,         ropuw,          "W   - Wolfram rule number ",  
	r_noopc,	False,		" X  - ",
	r_noopc,	False,		" Y  - ",
	r_noopc,	False,		" Z  - ",
	r_comment,	False,		" [  - ",
	r_noopc,	False,		" \    ",
	r_ubrack,	False,		" ]  - ",
	r_noopc,	False,		" ^  - ",
	r_code,		r_quit,		" _  - panic exit",
	r_noopc,	False,		" `  - ",
	r_noopc,	False,		" a  - ",
	r_noopc,	False,		" b  - ",
	r_noopc,	False,		" c  - ",
	r_noopc,	False,		" d  - ",
	r_noopc,	False,		" e  - ",
	r_noopc,	False,		" f  - ",
	r_noopc,	False,		" g  - ",
	r_noopc,	False,		" h  - ",
	r_noopc,	False,		" i  - ",
	r_noopc,	False,		" j  - ",
	r_noopc,	False,		" k  - ",
        r_pred,         ropll,          "l   - move cursor left if possible ",
	r_noopc,	False,		" m  - ",
	r_noopc,	False,		" n  - ",
	r_noopc,	False,		" o  - ",
	r_noopc,	False,		" p  - ",
	r_noopc,	False,		" q  - ",
        r_pred,         roplr,          "r   - move cursor right if possible",
	r_noopc,	False,		" s  - ",
	r_noopc,	False,		" t  - ",
	r_noopc,	False,		" u  - ",
	r_noopc,	False,		" v  - ",
	r_noopc,	False,		" w  - ",
	r_noopc,	False,		" x  - ",
	r_noopc,	False,		" y  - ",
        r_code,         roplz,          "z   - bring cursor to origin ",
	r_lbrace,	r_xbrace,	" {  - ",
	r_noopc,	False,		" |  - ",
	r_ubrace,	False,		" }  - ",
	r_noopc,	False,		" ~  - "
};
